;****** Servo Tester 2
;       Analog include file
;       Voltage/current menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.dseg

AnalogVoltage:          .byte 2         ; Unsigned
AnalogVoltageMin:       .byte 2
AnalogVoltageMax:       .byte 2
AnalogVoltageCalib:     .byte 2
AnalogCurrent:          .byte 2         ; SIGNED !
AnalogCurrentMin:       .byte 2
AnalogCurrentMax:       .byte 2
AnalogCurrentCalib:     .byte 2

AnalogCurrentZero:      .byte 2
AnalogCurrentType:      .byte 1 ; 0=Unipolar; 1=Bipolar+; 2=Bipolar-


.cseg

Analog_ServiceTable:
        jmp    Analog_Init
        jmp    Analog_DisplayMenu
        jmp    Analog_TakeControl
        jmp    Analog_Left
        jmp    Analog_Right
        jmp    Analog_Push
        jmp    Analog_UpdateDisplay
        jmp    Analog_ICP
        jmp    Analog_TimerTick
        jmp    Analog_ADComplete
        jmp    Analog_UpdateCursor
        jmp    Analog_OutputSent
        jmp    Analog_TachoUpdate
        jmp    Analog_Hold


Analog_Init:
        rcall   Analog_ResetCounters_w1
        ret


Analog_ResetCounters_w1:
        ser     w1
        sts     AnalogVoltageMin,w1
        sts     AnalogVoltageMin+1,w1
        ldi     w1,low(0x7fff)
        ldi     w2,high(0x7fff)
        sts     AnalogCurrentMin,w1
        sts     AnalogCurrentMin+1,w2
        clr     w1
        sts     AnalogVoltageMax,w1
        sts     AnalogVoltageMax+1,w1
        ldi     w1,low(0x8000)
        ldi     w2,high(0x8000)
        sts     AnalogCurrentMax,w1
        sts     AnalogCurrentMax+1,w2
        ret


Analog_UpdateInputs_All:
        in      w5,SREG
        cli
        lds     w1,ADCChannel0
        lds     w2,ADCChannel0+1
        lds     XL,ADCChannel1
        lds     XH,ADCChannel1+1
        out     SREG,w5
        sts     AnalogVoltage,w1
        sts     AnalogVoltage+1,w2
        lds     w5,AnalogCurrentType
        tst     w5
        breq    AUI_UnipolarCurrent
        lds     YL,AnalogCurrentZero
        lds     YH,AnalogCurrentZero+1
        sub     XL,YL                           ;Correct for current zero offset
        sbc     XH,YH
        cpi     w5,2
        brne    AUI_NotReversePolarity
        com     XL                      ;Two's complement X
        com     XH
        adiw    XH:XL,1
AUI_UnipolarCurrent:
AUI_NotReversePolarity:
        sts     AnalogCurrent,XL
        sts     AnalogCurrent+1,XH
        movw    w4:w3,XH:XL
        lds     XL,AnalogVoltageMin
        lds     XH,AnalogVoltageMin+1
        cp      w1,XL
        cpc     w2,XH
        brsh    AUI_NotLowVoltage
        sts     AnalogVoltageMin,w1
        sts     AnalogVoltageMin+1,w2
AUI_NotLowVoltage:
        lds     XL,AnalogVoltageMax
        lds     XH,AnalogVoltageMax+1
        cp      XL,w1
        cpc     XH,w2
        brsh    AUI_NotHighVoltage
        sts     AnalogVoltageMax,w1
        sts     AnalogVoltageMax+1,w2
AUI_NotHighVoltage:
        lds     XL,AnalogCurrentMin
        lds     XH,AnalogCurrentMin+1
        cp      w3,XL
        cpc     w4,XH
        brge    AUI_NotLowCurrent               ;Signed comparison
        sts     AnalogCurrentMin,w3
        sts     AnalogCurrentMin+1,w4
AUI_NotLowCurrent:
        lds     XL,AnalogCurrentMax
        lds     XH,AnalogCurrentMax+1
        cp      XL,w3
        cpc     XH,w4
        brge    AUI_NotHighCurrent              ;Signed comparison
        sts     AnalogCurrentMax,w3
        sts     AnalogCurrentMax+1,w4
AUI_NotHighCurrent:
        ret


Analog_DisplayMenu:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*Analog_DMenu1)
        ldi     ZH,high(2*Analog_DMenu1)
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Analog_DMenu2)
        ldi     ZH,high(2*Analog_DMenu2)
        call    LCD_WriteString_Z
        ret

Analog_DMenu1:  .db "  Voltage and   ",0,0
Analog_DMenu2:  .db "    current     ",0,0


Analog_TakeControl:
        clr     c1                      ;c1 = Page (0-2)
        ldi     c2,5                    ;c2 = Downcounter (5*100ms)
        ldi     w1,EVENT_Display        ;Request display update
        call    EventQ_PutEventIfNew
        ret


Analog_Left:
        tst     c1
        breq    AL_Exit
        dec     c1
AL_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


Analog_Right:
        cpi     c1,2
        brsh    AR_Exit
        inc     c1
AR_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


Analog_Push:
        tst     c1
        brne    AP_NotMenuExit
        ldi     w1,EVENT_ReleaseControl
        call    EventQ_PutEventInFront
        rjmp    AP_Exit
AP_NotMenuExit:
        rcall   Analog_ResetCounters_w1
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
AP_Exit:
        ret


Analog_UpdateDisplay:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        tst     c1
        brne    AUD_Not0
        ldi     ZL,low(2*Analog_MenuV)
        ldi     ZH,high(2*Analog_MenuV)
        call    LCD_WriteString_Z
        lds     XL,AnalogVoltage
        lds     XH,AnalogVoltage+1
        lds     YL,AnalogVoltageCalib
        lds     YH,AnalogVoltageCalib+1
        rcall   Analog_CalcAndWriteValue_All
        ldi     w1,'V'
        call    LCD_SendData
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Analog_MenuA)
        ldi     ZH,high(2*Analog_MenuA)
        call    LCD_WriteString_Z
        lds     XL,AnalogCurrent
        lds     XH,AnalogCurrent+1
        lds     YL,AnalogCurrentCalib
        lds     YH,AnalogCurrentCalib+1
        rcall   Analog_CalcAndWriteValue_All
        ldi     w1,'A'
        call    LCD_SendData
        rjmp    AUD_Exit
AUD_Not0:
        cpi     c1,1
        brne    AUD_Not1
        ldi     ZL,low(2*Analog_MenuV)
        ldi     ZH,high(2*Analog_MenuV)
        call    LCD_WriteString_Z
        lds     XL,AnalogVoltageMax
        lds     XH,AnalogVoltageMax+1
        lds     YL,AnalogVoltageCalib
        lds     YH,AnalogVoltageCalib+1
        rcall   Analog_CalcAndWriteValue_All
        ldi     w1,'V'
        call    LCD_SendData
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Analog_MenuMM)
        ldi     ZH,high(2*Analog_MenuMM)
        call    LCD_WriteString_Z
        lds     XL,AnalogVoltageMin
        lds     XH,AnalogVoltageMin+1
        lds     YL,AnalogVoltageCalib
        lds     YH,AnalogVoltageCalib+1
        rcall   Analog_CalcAndWriteValue_All
        ldi     w1,'V'
        call    LCD_SendData
        rjmp    AUD_Exit
AUD_Not1:
;        cpi     c1,2                   ; 2 is last possible menu (for now)
;        brne    AUD_Not2
        ldi     ZL,low(2*Analog_MenuA)
        ldi     ZH,high(2*Analog_MenuA)
        call    LCD_WriteString_Z
        lds     XL,AnalogCurrentMax
        lds     XH,AnalogCurrentMax+1
        lds     YL,AnalogCurrentCalib
        lds     YH,AnalogCurrentCalib+1
        rcall   Analog_CalcAndWriteValue_All
        ldi     w1,'A'
        call    LCD_SendData
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Analog_MenuMM)
        ldi     ZH,high(2*Analog_MenuMM)
        call    LCD_WriteString_Z
        lds     XL,AnalogCurrentMin
        lds     XH,AnalogCurrentMin+1
        lds     YL,AnalogCurrentCalib
        lds     YH,AnalogCurrentCalib+1
        rcall   Analog_CalcAndWriteValue_All
        ldi     w1,'A'
        call    LCD_SendData
;        rjmp    AUD_Exit
;AUD_Not2:
AUD_Exit:
        ret

Analog_MenuV:   .db LCD_CharRightArrow," Voltage",0
Analog_MenuA:   .db "  Current",0
Analog_MenuMM:  .db LCD_CharLeftArrow," max/min",0


Analog_CalcAndWriteValue_All:
                                        ;X contains the 13 bit value from A/D conversion (can handle 16 bit signed values)
                                        ;Y contains calibration factor
        clr     ZL                      ;Assume not negative
        tst     XH
        brpl    ACAWV_Calculate         ;Skip negative conversion
        ser     ZL                      ;Set negative indicator
        com     XL                      ;Two's complement X
        com     XH
        adiw    XH:XL,1
ACAWV_Calculate:
        call    mul16x16_32_w           ;Value * calibration
        movw    XH:XL,w4:w3             ;Get high word from result (= Result / 65536)
        mov     w6,ZL                   ;Save negative indicator
        clr     YL
        clr     YH
        ldi     ZL,low(100)
        ldi     ZH,high(100)
        call    div32_16u_uw1XYZ        ;Result / 100
        tst     w6                      ;Was original value negative?
        brpl    ACAWV_Positive          ;Nope positive
        cpi     XL,10                   ;One or two digit integer
        brsh    ACAWV_NegativeTwoDigit
        ldi     w1,' '
        call    LCD_SendData
        ldi     w1,'-'
        call    LCD_SendData
        call    Util_WriteU8_1d_XY
        rjmp    ACAWV_Fraction
ACAWV_NegativeTwoDigit:
        ldi     w1,'-'
        call    LCD_SendData
        call    Util_WriteU8_2d00_XY
        rjmp    ACAWV_Fraction
ACAWV_Positive:
        call    Util_WriteU8_3d_XY
;        rjmp    ACAWV_Fraction
ACAWV_Fraction:
        ldi     w1,'.'
        call    LCD_SendData
        movw    XH:XL,u2:u1
        call    Util_WriteU8_2d00_XY
        ret


Analog_ICP:
        ret


Analog_TimerTick:
        ret


Analog_ADComplete:
        dec     c2
        brne    AADC_Exit
        ldi     c2,5
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
AADC_Exit:
        ret


Analog_UpdateCursor:
        ret


Analog_OutputSent:
        ret


Analog_TachoUpdate:
        ret


Analog_Hold:
        ret


