;****** Servo Tester 2
;       Config include file
;       EEPROM control and setup menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    Config_AbsoluteMinPWM   = 1000
.equ    Config_AbsoluteMaxPWM   = 65000

.dseg
ConfigMagicWord: .byte 2
ConfigMinPW:     .byte 2
ConfigMaxPW:     .byte 2
ConfigAutosave:  .byte 1
ConfigExpoInc:   .byte 1

.equ    Config_MagicWord = 0x2877       ;Change each time config layout changes

.eseg
ConfigEEReserved:
        .db     0,0,0,0,0,0,0,0         ;Reserved block. Do not use
ConfigEEBlock:
        .dw     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0         ; 64 bytes reserved for configuration
        .dw     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.cseg

Config_AddressTable:                    ;SRAM addresses and byte count of values
        .dw     ConfigMagicWord, 2      ;to be saved in EEPROM
        .dw     ConfigMinPW, 2
        .dw     ConfigMaxPW, 2
        .dw     ConfigAutosave, 1
        .dw     RouterOutputA, 1
        .dw     RouterOutputB, 1
        .dw     Output1Value, 2
        .dw     Output2Value, 2
        .dw     Sweep1Min, 2
        .dw     Sweep1Max, 2
        .dw     Sweep1Step, 2
        .dw     Sweep2Min, 2
        .dw     Sweep2Max, 2
        .dw     Sweep2Step, 2
        .dw     ConfigExpoInc, 1
        .dw     TachoBlades, 1
        .dw     AnalogVoltageCalib, 2
        .dw     AnalogCurrentCalib, 2
        .dw     BacklightTime, 1
        .dw     RotaryReverse, 1
        .dw     AnalogCurrentZero, 2
        .dw     AnalogCurrentType, 1
        .dw     BatteryLowLimit, 2
        .dw     0       ;End marker

Config_DefaultValueTable:               ;Default values in case of corrupt EEPROM
        .dw     Config_MagicWord ;ConfigMagicWord
        .dw     9000            ;ConfigMinPW
        .dw     21000           ;ConfigMaxPW
        .dw     0               ;ConfigAutosave
        .dw     Router_Output1  ;RouterOutputA
        .dw     Router_Output2  ;RouterOutputB
        .dw     15000           ;Output1Value
        .dw     15000           ;Output2Value
        .dw     15000           ;Sweep1Min
        .dw     15000           ;Sweep1Max
        .dw     20              ;Sweep1Step
        .dw     13000           ;Sweep2Min
        .dw     17000           ;Sweep2Max
        .dw     20              ;Sweep2Step
        .dw     1               ;ConfigExpoInc
        .dw     2               ;TachoBlades
        .dw     8154            ;AnalogVoltageCalib
        .dw     4023            ;AnalogCurrentCalib
        .dw     0               ;BacklightTime (10 sec steps; 0 = no light; 0xff = always light)
        .dw     0               ;RotaryReverse
        .dw     4083            ;AnalogCurrentZero
        .dw     2               ;AnalogCurrentType
        .dw     440             ;BatteryLowLimit


.equ ConfigC2EditMode     = 0


Config_Init:                    ;Read entire configuration from EEPROM
        ldi     XL,low(ConfigEEBlock)
        ldi     XH,high(ConfigEEBlock)
        ldi     ZL,low(Config_AddressTable*2)
        ldi     ZH,high(Config_AddressTable*2)
CI_Loop:
        lpm     YL,Z+
        lpm     YH,Z+
        mov     w1,YL
        or      w1,YH
        breq    CI_Exit
        lpm     w1,Z+
        adiw    ZH:ZL,1
        rcall   Config_EEReadBlock_w123XY
        rjmp    CI_Loop
CI_Exit:
        lds     w1,ConfigMagicWord
        cpi     w1,low(Config_MagicWord)
        brne    CI_InvalidConfig
        lds     w1,ConfigMagicWord+1
        cpi     w1,high(Config_MagicWord)
        breq    CI_ConfigOk
CI_InvalidConfig:
        rcall   Config_RestoreDefaults
CI_ConfigOk:
        ret


Config_EEReadBlock_w123XY:      ;w1=counter; X=EEPROM start address; Y=RAM destination
        in      w2,SREG
        cli
CEERB_Loop:
        sbic    EECR,EEWE
        rjmp    CEERB_Loop
        out     EEARH,XH
        out     EEARL,XL
        sbi     EECR,EERE
        in      w3,EEDR
        st      Y+,w3
        adiw    XH:XL,1
        dec     w1
        brne    CEERB_Loop
        out     SREG,w2
        ret


Config_RestoreDefaults:
        ldi     ZL,low(Config_AddressTable*2)
        ldi     ZH,high(Config_AddressTable*2)
        ldi     XL,low(Config_DefaultValueTable*2)
        ldi     XH,high(Config_DefaultValueTable*2)
CRD_Loop:
        lpm     YL,Z+
        lpm     YH,Z+
        mov     w1,YL
        or      w1,YH
        breq    CRD_Exit
        lpm     w1,Z+
        adiw    ZH:ZL,1
        movw    w4:w3,ZH:ZL
        movw    ZH:ZL,XH:XL
        mov     w5,w1
CRD_InnerLoop:
        lpm     w2,Z+
        st      Y+,w2
        dec     w1
        brne    CRD_InnerLoop
        sbrc    w5,0                    ;Skip if even number of bytes read
        adiw    ZH:ZL,1                 ;Add 1 due to word alignment in flash
        movw    XH:XL,ZH:ZL
        movw    ZH:ZL,w4:w3
        rjmp    CRD_Loop
CRD_Exit:
        rcall   Config_SaveConfiguration
        ret


Config_GetMinPW_X:
        lds     XL,ConfigMinPW
        lds     XH,ConfigMinPW+1
        ret
        
        
Config_GetMaxPW_X:
        lds     XL,ConfigMaxPW
        lds     XH,ConfigMaxPW+1
        ret
        

Config_Autosave_w1:
        lds     w1,ConfigAutosave
        andi    w1,1
        cpi     w1,1
        ret


Config_ExpoInc_w1:
        lds     w1,ConfigExpoInc
        andi    w1,1
        cpi     w1,1
        ret


Config_EEUpdateBlock_w1234XY:   ;w1=counter; X=EEPROM start address; Y=RAM source
        in      w2,SREG         ;Get IRQ status
CEEUB_OuterLoop:
        out     SREG,w2         ;Restore IRQ statusfor each loop (in order not to
CEEUB_Loop:                     ; have interrupts disabled for too long)
        sbic    EECR,EEWE
        rjmp    CEEUB_Loop
        cli
        sbic    EECR,EEWE       ;Make sure no interrupt has started a new write
        rjmp    CEEUB_OuterLoop
        out     EEARH,XH
        out     EEARL,XL
        sbi     EECR,EERE
        in      w3,EEDR
        ld      w4,Y+
        cp      w3,w4
        breq    CEEUB_SkipWrite
        out     EEDR,w4
        sbi     EECR,EEMWE      ;Enable EEPROM write
        sbi     EECR,EEWE       ;Write new value to config
CEEUB_SkipWrite:
        adiw    XH:XL,1         ;Increment EEPROM address
        dec     w1
        brne    CEEUB_OuterLoop
        out     SREG,w2
        ret


Config_SaveConfiguration:
        ldi     XL,low(ConfigEEBlock)
        ldi     XH,high(ConfigEEBlock)
        ldi     ZL,low(Config_AddressTable*2)
        ldi     ZH,high(Config_AddressTable*2)
CSC_Loop:
        lpm     YL,Z+
        lpm     YH,Z+
        mov     w1,YL
        or      w1,YH
        breq    CSC_Exit
        lpm     w1,Z+
        adiw    ZH:ZL,1
        rcall   Config_EEUpdateBlock_w1234XY
        rjmp    CSC_Loop
CSC_Exit:
        ret


Config_ForceZBetweenXandY_w12:
        ld      w1,Z+
        ld      w2,Z+
        cp      w1,XL
        cpc     w2,XH
        brsh    CFZBXY_NotLow
        movw    w2:w1,XH:XL
        rjmp    CFZBXY_NotHigh
CFZBXY_NotLow:
        cp      YL,w1
        cpc     YH,w2
        brsh    CFZBXY_NotHigh
        movw    w2:w1,YH:YL
CFZBXY_NotHigh:
        st      -Z,w2
        st      -Z,w1
        ret
        

Config_CheckMinMax_All:
        rcall   Config_GetMaxPW_X
        movw    YH:YL,XH:XL
        rcall   Config_GetMinPW_X
        ldi     ZL,low(Output1Value)
        ldi     ZH,high(Output1Value)
        rcall   Config_ForceZBetweenXandY_w12
        ldi     ZL,low(Output2Value)
        ldi     ZH,high(Output2Value)
        rcall   Config_ForceZBetweenXandY_w12
        ldi     ZL,low(Sweep1Min)
        ldi     ZH,high(Sweep1Min)
        rcall   Config_ForceZBetweenXandY_w12
        ldi     ZL,low(Sweep1Max)
        ldi     ZH,high(Sweep1Max)
        rcall   Config_ForceZBetweenXandY_w12
        ldi     ZL,low(Sweep2Min)
        ldi     ZH,high(Sweep2Min)
        rcall   Config_ForceZBetweenXandY_w12
        ldi     ZL,low(Sweep2Max)
        ldi     ZH,high(Sweep2Max)
        rcall   Config_ForceZBetweenXandY_w12
        ret



;---------------------------MENU-------------------------

.equ    Config_LastNormalMenu = 10

Config_ServiceTable:
        jmp    ConfigMenu_Init
        jmp    ConfigMenu_DisplayMenu
        jmp    ConfigMenu_TakeControl
        jmp    ConfigMenu_Left
        jmp    ConfigMenu_Right
        jmp    ConfigMenu_Push
        jmp    ConfigMenu_UpdateDisplay
        jmp    ConfigMenu_ICP
        jmp    ConfigMenu_TimerTick
        jmp    ConfigMenu_ADComplete
        jmp    ConfigMenu_UpdateCursor
        jmp    ConfigMenu_OutputSent
        jmp    ConfigMenu_TachoUpdate
        jmp    ConfigMenu_Hold


ConfigMenu_Init:
        ret


ConfigMenu_DisplayMenu:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*Config_DMenu1)
        ldi     ZH,high(2*Config_DMenu1)
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Config_DMenu2)
        ldi     ZH,high(2*Config_DMenu2)
        call    LCD_WriteString_Z
        ret

Config_DMenu1:  .db "  Setup         ",0,0
Config_DMenu2:  .db "                ",0,0


ConfigMenu_TakeControl:
        ldi     c1,1
        clr     c2
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


ConfigMenu_Left:
        sbrc    c2,ConfigC2EditMode
        rjmp    CML_EditMode
        tst     c1
        breq    CML_Exit
        dec     c1
        rjmp    CML_Exit
CML_EditMode:
        cpi     c1,7
        breq    CML_Menu7
        movw    ZH:ZL,c4:c3
        ld      XL,Z+
        ld      XH,Z+
        call    Util_GetSpeed2Factor_w1
        clr     w2
        sub     XL,w1
        sbc     XH,w2
        brcc    CML_SaveValue
        clr     XL
        clr     XH
CML_SaveValue:
        st      -Z,XH
        st      -Z,XL
        rjmp    CML_Exit
CML_Menu7:
        lds     w1,BacklightTime
        tst     w1
        breq    CML_Exit
        cpi     w1,0xff
        brne    CML_DecBacklight
        ldi     w1,6            ; = 60 seconds max
        rjmp    CML_SaveBacklight
CML_DecBacklight:
        dec     w1
CML_SaveBacklight:
        sts     BacklightTime,w1
        call    Backlight_Activity_r01w12       ;Backlight config changed. Use new value
CML_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


ConfigMenu_Right:
        sbrc    c2,ConfigC2EditMode
        rjmp    CMR_EditMode
        cpi     c1,Config_LastNormalMenu
        brsh    CMR_SwitchToHiddenMenu
        inc     c1
        rjmp    CMR_Exit
CMR_EditMode:
        cpi     c1,7
        breq    CMR_Menu7
        movw    ZH:ZL,c4:c3
        ld      XL,Z+
        ld      XH,Z+
        call    Util_GetSpeed2Factor_w1
        clr     w2
        add     XL,w1
        adc     XH,w2
        brcc    CMR_SaveValue
        ser     XL
        ser     XH
CMR_SaveValue:
        st      -Z,XH
        st      -Z,XL
        rjmp    CMR_Exit
CMR_Menu7:
        lds     w1,BacklightTime
        cpi     w1,6            ; = 60 seconds max
        brlo    CMR_IncBacklight
        ser     w1              ; = always light on
        rjmp    CMR_SaveBacklight
CMR_IncBacklight:
        inc     w1
CMR_SaveBacklight:
        sts     BacklightTime,w1
        call    Backlight_Activity_r01w12       ;Backlight config changed. Use new value
        rjmp    CMR_Exit
CMR_SwitchToHiddenMenu:
        ldi     w1,EM_ConfigHidden_Menu
        sts     EventMan_FunctionInUse,w1       ;Swap menus silently
        call    ConfigH_TakeControl
CMR_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


ConfigMenu_Push:
        tst     c1
        brne    CMP_NotMenuExit
        rcall   Config_CheckMinMax_All
        ldi     w1,EVENT_ReleaseControl
        call    EventQ_PutEventInFront
        rjmp    CMP_Exit
CMP_NotMenuExit:
        cpi     c1,1
        brne    CMP_Not1
        lds     w1,ConfigAutosave
        ldi     w2,1
        eor     w1,w2
        sts     ConfigAutosave,w1
        rcall   Config_SaveConfiguration        ;Save config when toggling autosave
        rjmp    CMP_Exit
CMP_Not1:
        cpi     c1,2
        brne    CMP_Not2
        lds     w1,ConfigExpoInc
        ldi     w2,1
        eor     w1,w2
        sts     ConfigExpoInc,w1
        rjmp    CMP_Exit
CMP_Not2:
        cpi     c1,3
        brne    CMP_Not3
        ldi     w1,(1<<ConfigC2EditMode)
        eor     c2,w1
        ldi     c3,low(AnalogVoltageCalib)
        ldi     c4,high(AnalogVoltageCalib)
        rjmp    CMP_Exit
CMP_Not3:
        cpi     c1,4
        brne    CMP_Not4
        ldi     w1,(1<<ConfigC2EditMode)
        eor     c2,w1
        ldi     c3,low(AnalogCurrentZero)
        ldi     c4,high(AnalogCurrentZero)
        rjmp    CMP_Exit
CMP_Not4:
        cpi     c1,5
        brne    CMP_Not5
        ldi     w1,(1<<ConfigC2EditMode)
        eor     c2,w1
        ldi     c3,low(AnalogCurrentCalib)
        ldi     c4,high(AnalogCurrentCalib)
        rjmp    CMP_Exit
CMP_Not5:
        cpi     c1,6
        brne    CMP_Not6
        lds     w1,AnalogCurrentType
        inc     w1
        cpi     w1,3
        brlo    CMP6_NotWrapType
        clr     w1
CMP6_NotWrapType:
        sts     AnalogCurrentType,w1
        rjmp    CMP_Exit
CMP_Not6:
        cpi     c1,7
        brne    CMP_Not7
        ldi     w1,(1<<ConfigC2EditMode)
        eor     c2,w1
        rjmp    CMP_Exit
CMP_Not7:
        cpi     c1,8
        brne    CMP_Not8
        lds     w1,RotaryReverse
        ser     w2
        eor     w1,w2
        sts     RotaryReverse,w1
        rjmp    CMP_Exit
CMP_Not8:
        cpi     c1,9
        brne    CMP_Not9
        ldi     w1,(1<<ConfigC2EditMode)
        eor     c2,w1
        ldi     c3,low(BatteryLowLimit)
        ldi     c4,high(BatteryLowLimit)
        rjmp    CMP_Exit
CMP_Not9:
        cpi     c1,10
        brne    CMP_Not10
        rjmp    CMP_Exit                        ;Do nothing
CMP_Not10:
CMP_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


ConfigMenu_Hold:
        cpi     c1,10                           ;Test for Reset to defaults menu
        brne    CMH_Exit
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*Config_Reset1)
        ldi     ZH,high(2*Config_Reset1)
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Config_Reset2)
        ldi     ZH,high(2*Config_Reset2)
        call    LCD_WriteString_Z
        call    Config_RestoreDefaults
CMH_Exit:
        ret

Config_Reset1:  .db LCD_CharRightArrow," Default values",0,0
Config_Reset2:  .db LCD_CharLeftArrow," have been set ",0,0


ConfigMenu_UpdateDisplay:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*Config_MenuL1)
        ldi     ZH,high(2*Config_MenuL1)
        ldi     w1,18                           ;18 bytes between text lines
        mul     w1,c1
        add     ZL,r0
        adc     ZH,r1
        call    LCD_WriteString_Z               ;Write line 1
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        tst     c1
        brne    CMUD2_Not0
        ldi     ZL,low(2*Config_Menu02)
        ldi     ZH,high(2*Config_Menu02)
        rjmp    CMUD_Line2
CMUD2_Not0:
        cpi     c1,1
        brne    CMUD2_Not1
        rcall   Config_Autosave_w1
        rjmp    CMUD_SelectYesNo
CMUD2_Not1:
        cpi     c1,2
        brne    CMUD2_Not2
        rcall   Config_ExpoInc_w1
        rjmp    CMUD_SelectYesNo
CMUD2_Not2:
        cpi     c1,6
        brsh    CMUD2_Not345
        rjmp    CMUD2_345
CMUD2_Not345:
        cpi     c1,6
        brne    CMUD2_Not6
        rjmp    CMUD2_6
CMUD2_Not6:
        cpi     c1,7
        brne    CMUD2_Not7
        rjmp    CMUD2_7
CMUD2_Not7:
        cpi     c1,8
        brne    CMUD2_Not8
        lds     w1,RotaryReverse
        rjmp    CMUD_SelectYesNo
CMUD2_Not8:
        cpi     c1,9
        brne    CMUD2_Not9
        rjmp    CMUD2_9
CMUD2_Not9:
        cpi     c1,10
        brne    CMUD2_Not10
        rjmp    CMUD2_10
CMUD2_Not10:
        rjmp    CMUD_Exit
CMUD_SelectYesNo:
        tst     w1
        brne    CMUD_DisplayYes
        ldi     ZL,low(2*Config_MenuNo)
        ldi     ZH,high(2*Config_MenuNo)
        rjmp    CMUD_Line2
CMUD_DisplayYes:
        ldi     ZL,low(2*Config_MenuYes)
        ldi     ZH,high(2*Config_MenuYes)
;       rjmp    CMUD_Line2
CMUD_Line2:
        call    LCD_WriteString_Z
        rjmp    CMUD_Exit
CMUD2_6:
        ldi     ZL,low(2*Config_Menu6L2)
        ldi     ZH,high(2*Config_Menu6L2)
        ldi     w1,18                           ;18 bytes between text lines
        lds     w2,AnalogCurrentType
        mul     w1,w2
        add     ZL,r0
        adc     ZH,r1
        call    LCD_WriteString_Z               ;Write line 1
        rjmp    CMUD_Exit
CMUD2_7:
        ldi     w1,LCD_CharLeftArrow
        sbrc    c2,ConfigC2EditMode
        ldi     w1,LCD_CharAddSub
        call    LCD_SendData
        ldi     w1,' '
        call    LCD_SendData
        call    LCD_SendData
        call    LCD_SendData
        lds     w1,BacklightTime
        tst     w1
        breq    CMUD2_7_Off
        cpi     w1,0xff
        breq    CMUD2_7_On
        ori     w1,'0'
        call    LCD_SendData
        ldi     ZL,low(2*Config_Menu7Val)
        ldi     ZH,high(2*Config_Menu7Val)
        rjmp    CMUD2_7_WriteText
CMUD2_7_Off:
        ldi     ZL,low(2*Config_Menu7Off)
        ldi     ZH,high(2*Config_Menu7Off)
        rjmp    CMUD2_7_WriteText
CMUD2_7_On:
        ldi     ZL,low(2*Config_Menu7On)
        ldi     ZH,high(2*Config_Menu7On)
;       rjmp    CMUD2_7_WriteText
CMUD2_7_WriteText:
        call    LCD_WriteString_Z
        sbrs    c2,ConfigC2EditMode
        rjmp    CMUD_Exit
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     w1,' '
        call    LCD_SendData
        rjmp    CMUD_Exit
CMUD2_345:
        sbrc    c2,ConfigC2EditMode
        rjmp    CMUD2_345_EditMode
        ldi     ZL,low(2*Config_MenuClr)
        ldi     ZH,high(2*Config_MenuClr)
        call    LCD_WriteString_Z
        rjmp    CMUD_Exit
CMUD2_345_EditMode:
        ldi     w1,LCD_CharAddSub
        call    LCD_SendData
        ldi     w1,' '
        call    LCD_SendData
        movw    ZH:ZL,c4:c3
        ld      XL,Z+
        ld      XH,Z+
        call    Util_WriteU16_5d_XY
        lds     XL,AnalogVoltage
        lds     XH,AnalogVoltage+1
        lds     YL,AnalogVoltageCalib
        lds     YH,AnalogVoltageCalib+1
        cpi     c1,3
        breq    CMUD2_NotCurrent
        lds     XL,AnalogCurrent
        lds     XH,AnalogCurrent+1
        lds     YL,AnalogCurrentCalib
        lds     YH,AnalogCurrentCalib+1
CMUD2_NotCurrent:
        call    Analog_CalcAndWriteValue_All
        ldi     ZL,low(2*Config_MenuV)
        ldi     ZH,high(2*Config_MenuV)
        cpi     c1,3
        breq    CMUD2_NotCurrent2
        ldi     ZL,low(2*Config_MenuA)
        ldi     ZH,high(2*Config_MenuA)
CMUD2_NotCurrent2:
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     w1,' '
        call    LCD_SendData
        rjmp    CMUD_Exit
CMUD2_9:
        ldi     w1,LCD_CharLeftArrow
        sbrc    c2,ConfigC2EditMode
        ldi     w1,LCD_CharAddSub
        call    LCD_SendData
        ldi     w1,' '
        call    LCD_SendData
        call    LCD_SendData
        lds     XL,BatteryLowLimit
        lds     XH,BatteryLowLimit+1
        clr     YL
        clr     YH
        ldi     ZL,low(100)
        ldi     ZH,high(100)
        call    div32_16u_uw1XYZ        ;Result / 100
        call    Util_WriteU8_3d_XY
        ldi     w1,'.'
        call    LCD_SendData
        movw    XH:XL,u2:u1
        call    Util_WriteU8_2d00_XY
        ldi     ZL,low(2*Config_Menu92)
        ldi     ZH,high(2*Config_Menu92)
        call    LCD_WriteString_Z
        sbrs    c2,ConfigC2EditMode
        rjmp    CMUD_Exit
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     w1,' '
        call    LCD_SendData
        rjmp    CMUD_Exit
CMUD2_10:
        ldi     ZL,low(2*Config_MenuClr)
        ldi     ZH,high(2*Config_MenuClr)
        call    LCD_WriteString_Z
;       rjmp    CMUD_Exit
CMUD_Exit:
        ret

;Keep these lines together (and always 18 chars long)
Config_MenuL1:
        .db LCD_CharRightArrow," Exit Setup?   ",0,0
        .db LCD_CharRightArrow," Autosave?     ",0,0
        .db LCD_CharRightArrow," Exponential?  ",0,0
        .db LCD_CharRightArrow," Adj volt gain ",0,0
        .db LCD_CharRightArrow," Adj zero curr ",0,0
        .db LCD_CharRightArrow," Adj curr gain ",0,0
        .db LCD_CharRightArrow," Current input ",0,0
        .db LCD_CharRightArrow," Backlight time",0,0
        .db LCD_CharRightArrow," Knob reverse? ",0,0
        .db LCD_CharRightArrow," Low battery   ",0,0
        .db LCD_CharRightArrow," Reset setup   ",0,0

Config_Menu02:  .db "  Push to exit  ",0,0
Config_MenuNo:  .db LCD_CharLeftArrow,"    No         ",0,0
Config_MenuYes: .db LCD_CharLeftArrow,"    Yes        ",0,0
Config_MenuClr: .db LCD_CharLeftArrow,"               ",0,0
Config_MenuV:   .db " V ",0
Config_MenuA:   .db " A ",0
Config_Menu7Val: .db "0 seconds  ",0
Config_Menu7Off: .db "Always off  ",0,0
Config_Menu7On:  .db "Always on   ",0,0
Config_Menu92:  .db " V     ",0
Config_Menu6L2:
        .db LCD_CharLeftArrow," Unipolar      ",0,0
        .db LCD_CharLeftArrow," Bipolar +     ",0,0
        .db LCD_CharLeftArrow," Bipolar -     ",0,0


ConfigMenu_ICP:
        ret


ConfigMenu_TimerTick:
        sbrs    c2,ConfigC2EditMode
        rjmp    CMTT_Exit
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
CMTT_Exit:
        ret


ConfigMenu_ADComplete:
        ret


ConfigMenu_UpdateCursor:
        ret


ConfigMenu_OutputSent:
        ret


ConfigMenu_TachoUpdate:
        ret




;------------CONFIG HIDDEN MENU----------------

.dseg

ConfigHPage:    .byte 1


.cseg

ConfigH_ServiceTable:
        jmp    ConfigH_Init
        jmp    ConfigH_DisplayMenu
        jmp    ConfigH_TakeControl
        jmp    PWIO_Left
        jmp    PWIO_Right
        jmp    PWIO_Push
        jmp    PWIO_UpdateDisplay
        jmp    PWIO_ICP
        jmp    PWIO_TimerTick
        jmp    PWIO_ADComplete
        jmp    PWIO_UpdateCursor
        jmp    PWIO_OutputSent
        jmp    PWIO_TachoUpdate
        jmp    PWIO_Hold


ConfigH_Init:
        ret


ConfigH_InitPtrs:
        ldi     ZL,low(ConfigMinPW)
        ldi     ZH,high(ConfigMinPW)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*ConfigH_Page1)
        ldi     ZH,high(2*ConfigH_Page1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        ldi     ZL,low(ConfigH_ExtendedCallback)
        ldi     ZH,high(ConfigH_ExtendedCallback)
        sts     PWIOExtendedCallback,ZL
        sts     PWIOExtendedCallback+1,ZH
        clr     w1
        sts     ConfigHPage,w1
        ldi     w1,low(Config_AbsoluteMinPWM)
        ldi     w2,high(Config_AbsoluteMinPWM)
        sts     PWIOMinPW,w1
        sts     PWIOMinPW+1,w2
        call    Config_GetMaxPW_X
        sts     PWIOMaxPW,XL            ;Make sure max does not go below min
        sts     PWIOMaxPW+1,XH
        ldi     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3ExtendedMenu)
        ret


ConfigH_DisplayMenu:
        ret             ;Hidden menu can't be displayed


ConfigH_TakeControl:    ;Call manually
        rcall   ConfigH_InitPtrs
        call    PWIO_TakeControl
        ldi     c1,1    ;Cheat a bit (move cursor to pos 1)
        ret


ConfigH_Page1:  .db "  PWM low limit ",0,0
ConfigH_Page2:  .db "  PWM high limit",0,0


ConfigH_ExtendedCallback:
        lds     w2,ConfigHPage
        cpi     w1,PWIO_EXTENDLEFT
        brne    CHEC_Right
        tst     w2
        brne    CHEC_GoLeft
        rjmp    CHEC_ReturnToNormalMenu
CHEC_GoLeft:
        dec     w2
        rjmp    CHEC_SetupPage
CHEC_Right:
        cpi     w2,1
        brne    CHEC_GoRight
        rjmp    CHEC_Exit
CHEC_GoRight:
        inc     w2
CHEC_SetupPage:
        sts     ConfigHPage,w2
        cpi     w2,0
        brne    CHEC_NotPage1
        ldi     ZL,low(ConfigMinPW)
        ldi     ZH,high(ConfigMinPW)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*ConfigH_Page1)
        ldi     ZH,high(2*ConfigH_Page1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        ldi     XL,low(Config_AbsoluteMinPWM)
        ldi     XH,high(Config_AbsoluteMinPWM)
        sts     PWIOMinPW,XL
        sts     PWIOMinPW+1,XH
        call    Config_GetMaxPW_X
        sts     PWIOMaxPW,XL            ;Make sure max does not go below min
        sts     PWIOMaxPW+1,XH
        ldi     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3ExtendedMenu)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    CHEC_Page1NotLeft
        ldi     c1,5
CHEC_Page1NotLeft:
        rjmp    CHEC_Exit
CHEC_NotPage1:
        cpi     w2,1
        brne    CHEC_Exit               ;Unknown page !!
        ldi     ZL,low(ConfigMaxPW)
        ldi     ZH,high(ConfigMaxPW)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*ConfigH_Page2)
        ldi     ZH,high(2*ConfigH_Page2)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    Config_GetMinPW_X
        sts     PWIOMinPW,XL            ;Make sure min does not exceed max
        sts     PWIOMinPW+1,XH
        ldi     XL,low(Config_AbsoluteMaxPWM)
        ldi     XH,high(Config_AbsoluteMaxPWM)
        sts     PWIOMaxPW,XL
        sts     PWIOMaxPW+1,XH
        ldi     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3ExtendedMenu)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    CHEC_Page2NotLeft
        ldi     c1,5
CHEC_Page2NotLeft:
        rjmp    CHEC_Exit
CHEC_ReturnToNormalMenu:
        call    Cursor_Stop
        ldi     w1,EM_Config_Menu
        sts     EventMan_FunctionInUse,w1       ;Swap menus silently
        ldi     c1,Config_LastNormalMenu        ;Enter at last normal menu
        clr     c2
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
CHEC_Exit:
        ret


