;****** Servo Tester 2
;       EventMan include file
;       Event Manager
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    EventMan_OffsetInit             = 0
.equ    EventMan_OffsetDisplayMenu      = 1
.equ    EventMan_OffsetTakeControl      = 2
.equ    EventMan_OffsetLeft             = 3
.equ    EventMan_OffsetRight            = 4
.equ    EventMan_OffsetPush             = 5
.equ    EventMan_OffsetUpdateDisplay    = 6
.equ    EventMan_OffsetICP              = 7
.equ    EventMan_OffsetTimerTick        = 8
.equ    EventMan_OffsetADComplete       = 9
.equ    EventMan_OffsetUpdateCursor     = 10
.equ    EventMan_OffsetOutputSent       = 11
.equ    EventMan_OffsetTachoUpdate      = 12
.equ    EventMan_OffsetHold             = 13


.cseg

EventMan_RegisteredMenus:

.set Counter = 1
.equ EM_Output1_Menu = Counter
.dw  Output1_ServiceTable

.set Counter = Counter+1
.equ EM_Output2_Menu = Counter
.dw  Output2_ServiceTable

.set Counter = Counter+1
.equ EM_Input_Menu = Counter
.dw  Input_ServiceTable

.set Counter = Counter+1
.equ EM_InputAvg_Menu = Counter
.dw  InputAvg_ServiceTable

.set Counter = Counter+1
.equ EM_InputStat_Menu = Counter
.dw  InputStat_ServiceTable

.set Counter = Counter+1
.equ EM_Sweep1_Menu = Counter
.dw  Sweep1_ServiceTable

.set Counter = Counter+1
.equ EM_Sweep2_Menu = Counter
.dw  Sweep2_ServiceTable

.set Counter = Counter+1
.equ EM_Analog_Menu = Counter
.dw  Analog_ServiceTable

.set Counter = Counter+1
.equ EM_Tacho_Menu = Counter
.dw  Tacho_ServiceTable

.set Counter = Counter+1
.equ EM_Idle_Menu = Counter
.dw  Idle_ServiceTable

.set Counter = Counter+1
.equ EM_Config_Menu = Counter
.dw  Config_ServiceTable


.equ EventMan_NumVisibleMenus = Counter

;The following menus are hidden

.set Counter = Counter+1
.equ EM_ConfigHidden_Menu = Counter
.dw  ConfigH_ServiceTable

.equ EventMan_NumAllMenus = Counter


.dseg
EventMan_FunctionInUse: .byte 1         ; 0=Menu navigation, No menu active
EventMan_MenuDisplayed: .byte 1         ; 1-max (0 is not used!)
EventMan_Ticks:         .byte 1



.cseg
EventMan_Init:
        call    EventQ_Init
        ldi     w1,10
        sts     EventMan_Ticks,w1
        clr     w1
        sts     EventMan_FunctionInUse,w1
        ldi     w1,1
        sts     EventMan_MenuDisplayed,w1
        ldi     w1,EventMan_NumAllMenus
        ldi     ZL,low(EventMan_RegisteredMenus*2)
        ldi     ZH,high(EventMan_RegisteredMenus*2)
EMI_Loop:
        lpm     YL,Z+                   ;Call all Inits from the menus
        lpm     YH,Z+
        adiw    YH:YL,EventMan_OffsetInit
        push    ZL
        push    ZH
        push    w1
        movw    ZH:ZL,YH:YL
        icall
        pop     w1
        pop     ZH
        pop     ZL
        dec     w1
        brne    EMI_Loop
        ret


EventMan_Update:
        call    EventQ_GetEvent_w1      ;Read next event
        brts    EMU_NewEvent
        ret
EMU_NewEvent:                           ;New event received. Find out which one is is
        lds     w2,EventMan_FunctionInUse
        cpi     w1,EVENT_Left
        brne    EMU_NotLeft
        rjmp    EM_Left
EMU_NotLeft:
        cpi     w1,EVENT_Right
        brne    EMU_NotRight
        rjmp    EM_Right
EMU_NotRight:
        cpi     w1,EVENT_Push
        brne    EMU_NotPush
        rjmp    EM_Push
EMU_NotPush:
        cpi     w1,EVENT_Display
        brne    EMU_NotDisplay
        rjmp    EM_Display
EMU_NotDisplay:
        cpi     w1,EVENT_ReleaseControl
        brne    EMU_NotReleaseControl
        rjmp    EM_ReleaseControl
EMU_NotReleaseControl:
        cpi     w1,EVENT_ICP
        brne    EMU_NotICP
        rjmp    EM_ICP
EMU_NotICP:
        cpi     w1,EVENT_TimerTick
        brne    EMU_NotTimerTick
        rjmp    EM_TimerTick
EMU_NotTimerTick:
        cpi     w1,EVENT_ADComplete
        brne    EMU_NotADComplete
        rjmp    EM_ADComplete
EMU_NotADComplete:
        cpi     w1,EVENT_UpdateCursor
        brne    EMU_NotUpdateCursor
        rjmp    EM_UpdateCursor
EMU_NotUpdateCursor:
        cpi     w1,EVENT_OutputSent
        brne    EMU_NotOutputSent
        rjmp    EM_OutputSent
EMU_NotOutputSent:
        cpi     w1,EVENT_TachoUpdate
        brne    EMU_NotTachoUpdate
        rjmp    EM_TachoUpdate
EMU_NotTachoUpdate:
        cpi     w1,EVENT_Hold
        brne    EMU_NotHold
        rjmp    EM_Hold
EMU_NotHold:
;Possible error handler here. Event not recognised.
        ret


EM_Left:                ;User has turned the knob a click counterclockwise
        call    Backlight_Activity_r01w12       ;Turn on backlight at user activity
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMLeft_InControl
        ldi     w1,EventMan_OffsetLeft
        rjmp    EM_CallService
EMLeft_InControl:
        lds     w2,EventMan_MenuDisplayed
        dec     w2
        brne    EMLeft_NotEndstop
        inc     w2
EMLeft_NotEndstop:
        sts     EventMan_MenuDisplayed,w2
        ldi     w1,EVENT_Display
        call    EventQ_PutEventIfNew
        ret


EM_Right:               ;User has turned the knob a click clockwise
        call    Backlight_Activity_r01w12       ;Turn on backlight at user activity
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMRight_InControl
        ldi     w1,EventMan_OffsetRight
        rjmp    EM_CallService
EMRight_InControl:
        lds     w2,EventMan_MenuDisplayed
        cpi     w2,EventMan_NumVisibleMenus
        brsh    EMRight_Endstop
        inc     w2
EMRight_Endstop:
        sts     EventMan_MenuDisplayed,w2
        ldi     w1,EVENT_Display
        call    EventQ_PutEventIfNew
        ret


EM_Push:                ;User has pushed down on the knob
        call    Backlight_Activity_r01w12       ;Turn on backlight at user activity
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMPush_InControl
        ldi     w1,EventMan_OffsetPush
        rjmp    EM_CallService
EMPush_InControl:
        lds     w2,EventMan_MenuDisplayed
        sts     EventMan_FunctionInUse,w2
        ldi     w1,EventMan_OffsetTakeControl
        rjmp    EM_CallService


EM_Display:             ;Display update requested
        tst     w2
        breq    EMDisplay_InControl
        ldi     w1,EventMan_OffsetUpdateDisplay
        rcall   EM_CallService
        call    Battery_Low_w1
        brne    EMDisplay_Exit
        rcall   EventMan_FlashLowBat
        rjmp    EMDisplay_Exit
EMDisplay_InControl:
        lds     w2,EventMan_MenuDisplayed
        ldi     w1,EventMan_OffsetDisplayMenu
        rcall   EM_CallService
        call    Battery_Low_w1
        brne    EMDisplay_NoLowBat
        rcall   EventMan_FlashLowBat
        rjmp    EMDisplay_NoUpArrow
EMDisplay_NoLowBat:
        lds     w3,EventMan_MenuDisplayed
        cpi     w3,1
        breq    EMDisplay_NoUpArrow
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     w1,LCD_CharUpArrow
        call    LCD_SendData
EMDisplay_NoUpArrow:
        cpi     w3,EventMan_NumVisibleMenus
        breq    EMDisplay_NoDownArrow
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     w1,LCD_CharDownArrow
        call    LCD_SendData
EMDisplay_NoDownArrow:
EMDisplay_Exit:
        ret


EM_ReleaseControl:      ;Submenu is exiting and returning control to the eventmanager
        call    Cursor_Stop
        clr     w2
        sts     EventMan_FunctionInUse,w2
        call    Config_Autosave_w1
        brne    EMRC_NoSave
        call    Config_SaveConfiguration
EMRC_NoSave:
        ldi     w1,EVENT_Display
        call    EventQ_PutEventIfNew
        ret


EM_ICP:                 ;New Input Capture value
        in      w1,SREG
        cli
        lds     XL,PulseInput
        lds     XH,PulseInput+1
        lds     YL,PulseFrameLen
        lds     YH,PulseFrameLen+1
        lds     ZL,PulseFrameLen+2
        out     SREG,w1
        sts     InputValue,XL
        sts     InputValue+1,XH
        sts     InputStatFrameLen,YL
        sts     InputStatFrameLen+1,YH
        sts     InputStatFrameLen+2,ZL
        call    InputAvg_UpdateBuffer_All
        lds     XL,InputValue
        lds     XH,InputValue+1
        call    InputStat_UpdateInputs_All
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMICP_InControl
        ldi     w1,EventMan_OffsetICP
        rjmp    EM_CallService
EMICP_InControl:
        lds     w1,EventMan_MenuDisplayed
        cpi     w1,EM_Input_Menu
        breq    EMICP_UpdateDisplay
        cpi     w1,EM_InputAvg_Menu
        brne    EMICP_Exit
EMICP_UpdateDisplay:
        ldi     w1,EVENT_Display
        call    EventQ_PutEventIfNew
EMICP_Exit:
        ret


EM_TimerTick:           ;Timer tick. Comes regularly each 100ms
        call    Cursor_TimerTick_All            ;Always forward timer ticks to cursor
        call    ADC_StartConversion_All         ;Start A/D conversion 10 times per second
        call    Backlight_TimerTick_All         ;Always forward timer ticks to backlight
        rcall   EventMan_TimerTick_All
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMTimer_InControl
        ldi     w1,EventMan_OffsetTimerTick
        rjmp    EM_CallService
EMTimer_InControl:
        ret     ;We have no use for timer ticks in the menu (yet)


EM_ADComplete:          ;A/D converter has finished
        call    Analog_UpdateInputs_All         ;Update voltage and current measurements
        call    Battery_UpdateVoltage_All
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMADC_InControl
        ldi     w1,EventMan_OffsetADComplete
        rjmp    EM_CallService
EMADC_InControl:
        ret     ;We have no use for A/D values in the menu


EM_UpdateCursor:        ;Display has changed and cursor needs updating
        tst     w2
        breq    EMUC_InControl
        ldi     w1,EventMan_OffsetUpdateCursor
        rjmp    EM_CallService
EMUC_InControl:
        ret     ;We have no use for cursor updates in the menu


EM_OutputSent:          ;New pulses have been sent to servo outputs
        call    Sweep1_UpdateSweep_All      ; Update sweep outputs
        call    Sweep2_UpdateSweep_All
        lds     w2,EventMan_FunctionInUse
        tst     w2
        breq    EMOS_InControl
        ldi     w1,EventMan_OffsetOutputSent
        rjmp    EM_CallService
EMOS_InControl:
        lds     w1,EventMan_MenuDisplayed
        cpi     w1,EM_Sweep1_Menu
        breq    EMOS_UpdateDisplay
        cpi     w1,EM_Sweep2_Menu
        brne    EMOS_Exit
EMOS_UpdateDisplay:
        ldi     w1,EVENT_Display
        call    EventQ_PutEventIfNew
EMOS_Exit:
        ret


EM_TachoUpdate:         ;New measurement from tachometer
        tst     w2
        breq    EMTU_InControl
        ldi     w1,EventMan_OffsetTachoUpdate
        rjmp    EM_CallService
EMTU_InControl:
        lds     w1,EventMan_MenuDisplayed
        cpi     w1,EM_Tacho_Menu
        brne    EMTU_Exit
        ldi     w1,EVENT_Display
        call    EventQ_PutEventIfNew
EMTU_Exit:
        ret


EM_Hold:                ;User is holding the button down
        tst     w2
        breq    EMH_InControl
        ldi     w1,EventMan_OffsetHold
        rjmp    EM_CallService
EMH_InControl:
        ret     ;We have no use for hold indications in the menu



; w2 = Which function call-block to use (1-n)
; w1 = Which function inside block (0-x)
EM_CallService:
        tst     w2
        breq    EMCS_Error
        dec     w2
        cpi     w2,EventMan_NumAllMenus
        brsh    EMCS_Error
        clr     w3
        lsl     w2
        rol     w3
        ldi     ZL,low(EventMan_RegisteredMenus*2)
        ldi     ZH,high(EventMan_RegisteredMenus*2)
        add     ZL,w2
        adc     ZH,w3
        lpm     YL,Z+
        lpm     YH,Z+
        mov     ZL,w1
        clr     ZH
        lsl     ZL
        rol     ZH
        add     ZL,YL
        adc     ZH,YH
        icall
        ret
EMCS_Error:
;Possible error handler here. Params outside boundaries
        ret


EventMan_FlashLowBat:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     w1,'B'
        lds     w2,EventMan_Ticks
        cpi     w2,6
        brsh    EMFLB_Display
        ldi     w1,' '
EMFLB_Display:
        call    LCD_SendData
        ret
        

EventMan_TimerTick_All:
        lds     w1,EventMan_Ticks
        dec     w1
        brne    EMTT_Save
        ldi     w1,10
EMTT_Save:
        sts     EventMan_Ticks,w1
        cpi     w1,10
        breq    EMTT_Update
        cpi     w1,5
        brne    EMTT_Exit
EMTT_Update:
        call    Battery_Low_w1
        brne    EMTT_Exit
        rcall   EventMan_FlashLowBat
EMTT_Exit:
        ret

