;****** Servo Tester 2
;       EventQ include file
;       EventQueue
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


;EventQueue functions can be called from interrupts

.equ    EventQueueSize  = 32

.dseg
EventQueue:     .byte EventQueueSize
EventQInPtr:    .byte 2
EventQOutPtr:   .byte 2
EventQCount:      .byte 1

.cseg

EventQ_Init:
        ldi     w1,low(EventQueue)
        ldi     w2,high(EventQueue)
        sts     EventQInPtr,w1
        sts     EventQInPtr+1,w2
        sts     EventQOutPtr,w1
        sts     EventQOutPtr+1,w2
        clr     w1
        sts     EventQCount,w1
        ret


EventQ_PutEvent:        ;w1 = event
        push    w2
        push    w3
        push    ZL
        push    ZH
        in      w3,SREG
        cli
        lds     w2,EventQCount
        cpi     w2,EventQueueSize
        brsh    EQPE_Exit
        inc     w2
        sts     EventQCount,w2
        lds     ZL,EventQInPtr
        lds     ZH,EventQInPtr+1
        st      Z+,w1
        cpi     ZL,low(EventQueue+EventQueueSize)
        brne    EQPE_NoWrap
        cpi     ZH,high(EventQueue+EventQueueSize)
        brne    EQPE_NoWrap
        ldi     ZL,low(EventQueue)
        ldi     ZH,high(EventQueue)
EQPE_NoWrap:
        sts     EventQInPtr,ZL
        sts     EventQInPtr+1,ZH
EQPE_Exit:
        out     SREG,w3
        pop     ZH
        pop     ZL
        pop     w3
        pop     w2
        ret


EventQ_PutEventIfNew:   ;w1 = event
        push    w2
        push    w3
        push    w4
        push    ZL
        push    ZH
        in      w4,SREG
        cli
        lds     w2,EventQCount
        tst     w2
        breq    EQPEIN_PutInQueue
        lds     ZL,EventQOutPtr
        lds     ZH,EventQOutPtr+1
EQPEIN_Loop:
        ld      w3,Z+
        cp      w1,w3
        breq    EQPEIN_Exit
        cpi     ZL,low(EventQueue+EventQueueSize)
        brne    EQPEIN_NoWrap
        cpi     ZH,high(EventQueue+EventQueueSize)
        brne    EQPEIN_NoWrap
        ldi     ZL,low(EventQueue)
        ldi     ZH,high(EventQueue)
EQPEIN_NoWrap:
        dec     w2
        brne    EQPEIN_Loop
EQPEIN_PutInQueue:
        rcall   EventQ_PutEvent
EQPEIN_Exit:
        out     SREG,w4
        pop     ZH
        pop     ZL
        pop     w4
        pop     w3
        pop     w2
        ret


EventQ_PutEventInFront: ;w1 = event
        push    w2
        push    w3
        push    ZL
        push    ZH
        in      w3,SREG
        cli
        lds     w2,EventQCount
        cpi     w2,EventQueueSize
        brsh    EQPEIF_Exit
        inc     w2
        sts     EventQCount,w2
        lds     ZL,EventQOutPtr
        lds     ZH,EventQOutPtr+1
        cpi     ZL,low(EventQueue)
        brne    EQPEIF_NoWrap
        cpi     ZH,high(EventQueue)
        brne    EQPEIF_NoWrap
        ldi     ZL,low(EventQueue+EventQueueSize)
        ldi     ZH,high(EventQueue+EventQueueSize)
EQPEIF_NoWrap:
        st      -Z,w1
        sts     EventQOutPtr,ZL
        sts     EventQOutPtr+1,ZH
EQPEIF_Exit:
        out     SREG,w3
        pop     ZH
        pop     ZL
        pop     w3
        pop     w2
        ret


EventQ_GetEvent_w1:     ;w1 = event  T=0 ->no event ( -> w1 undefined )
        push    w2
        push    ZL
        push    ZH
        clt
        in      w2,SREG
        cli
        lds     w1,EventQCount
        tst     w1
        breq    EQGE_Exit
        dec     w1
        sts     EventQCount,w1
        lds     ZL,EventQOutPtr
        lds     ZH,EventQOutPtr+1
        ld      w1,Z+
        cpi     ZL,low(EventQueue+EventQueueSize)
        brne    EQGE_NoWrap
        cpi     ZH,high(EventQueue+EventQueueSize)
        brne    EQGE_NoWrap
        ldi     ZL,low(EventQueue)
        ldi     ZH,high(EventQueue)
EQGE_NoWrap:
        sts     EventQOutPtr,ZL
        sts     EventQOutPtr+1,ZH
        sbr     w2,(1<<6)               ;Set T flag when restoring SREG
EQGE_Exit:
        out     SREG,w2
        pop     ZH
        pop     ZL
        pop     w2
        ret


