;****** Servo Tester 2
;       InputAvg include file
;       Input Average menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    AVG_SIZEEXP     =5              ;Averaging buffer size. 2^n elements
.equ    AVG_ELEMENTS    =(1<<AVG_SIZEEXP)       ; = 2^5 = 32 elements (words)
.equ    AVG_BUFFERSIZE  =2*AVG_ELEMENTS         ; = 64 bytes

.dseg
InputAvgValue:  .byte 2         ; Calculated value
InputAvgSum:    .byte 3
InputAvgBuffer: .byte AVG_BUFFERSIZE
InputAvgBufPtr: .byte 2


.cseg

InputAvg_ServiceTable:
        jmp    InputAvg_Init
        jmp    InputAvg_DisplayMenu
        jmp    InputAvg_TakeControl
        jmp    PWIO_Left
        jmp    PWIO_Right
        jmp    PWIO_Push
        jmp    PWIO_UpdateDisplay
        jmp    InputAvg_ICP
        jmp    PWIO_TimerTick
        jmp    PWIO_ADComplete
        jmp    PWIO_UpdateCursor
        jmp    PWIO_OutputSent
        jmp    PWIO_TachoUpdate
        jmp    PWIO_Hold


InputAvg_Init:
        clr     w1
        sts     InputAvgValue,w1
        sts     InputAvgValue+1,w1
        sts     InputAvgSum,w1
        sts     InputAvgSum+1,w1
        sts     InputAvgSum+2,w1
        ldi     ZL,low(InputAvgBuffer)
        ldi     ZH,high(InputAvgBuffer)
        sts     InputAvgBufPtr,ZL
        sts     InputAvgBufPtr+1,ZH
        ldi     w2,AVG_BUFFERSIZE
IAI_Loop:
        st      Z+,w1
        dec     w2
        brne    IAI_Loop
        ret


InputAvg_InitPtrs:
        ldi     ZL,low(InputAvgValue)
        ldi     ZH,high(InputAvgValue)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*InputAvg_Line1)
        ldi     ZH,high(2*InputAvg_Line1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        ldi     w1,Router_InputAvg
        sts     PWIORouterAdr,w1
        call    PWIO_SetupPWConfigLimit_X
        clr     c2
        ldi     c3,(1<<PWIOC3DisplayAB)
        ret


InputAvg_DisplayMenu:
        rcall   InputAvg_InitPtrs
        call    PWIO_BaseDisplay
        ret


InputAvg_TakeControl:
        rcall   InputAvg_InitPtrs
        call    PWIO_TakeControl
        ret


InputAvg_ICP:
        ldi     w1,2
        ldi     w2,1
        call    LCD_GotoXY
        lds     XL,InputAvgValue
        lds     XH,InputAvgValue+1
        call    Util_WritePulseWidth_XY
        ret

InputAvg_Line1: .db "  Input avg.  AB",0,0



InputAvg_UpdateBuffer_All:              ;New ICP value in X
        lds     w1,InputAvgSum          ;Get sum of entire buffer
        lds     w2,InputAvgSum+1
        lds     w3,InputAvgSum+2
        lds     ZL,InputAvgBufPtr
        lds     ZH,InputAvgBufPtr+1
        ld      YL,Z+                   ;Get old buffervalue
        ld      YH,Z+
        clr     w4
        sub     w1,YL                   ;Subtract old buffervalue
        sbc     w2,YH
        sbc     w3,w4
        add     w1,XL                   ;Add new buffervalue
        adc     w2,XH
        adc     w3,w4
        st      -Z,XH                   ;Save new buffervalue
        st      -Z,XL
        sts     InputAvgSum,w1          ;Save sum of entire buffer
        sts     InputAvgSum+1,w2
        sts     InputAvgSum+2,w3
        ldi     w4,AVG_SIZEEXP          ;Prepare to divide
IAUB_DivideLoop:
        lsr     w3                      ;Divide sum to find average pulsewidth
        ror     w2
        ror     w1
        dec     w4
        brne    IAUB_DivideLoop
        sts     InputAvgValue,w1        ;Save average pulsewidth
        sts     InputAvgValue+1,w2
        adiw    ZH:ZL,2
        ldi     YL,low(InputAvgBuffer+AVG_BUFFERSIZE)
        ldi     YH,high(InputAvgBuffer+AVG_BUFFERSIZE)
        cp      ZL,YL
        cpc     ZH,YH
        brlo    IAUB_NoWrap
        ldi     ZL,low(InputAvgBuffer)
        ldi     ZH,high(InputAvgBuffer)
IAUB_NoWrap:
        sts     InputAvgBufPtr,ZL
        sts     InputAvgBufPtr+1,ZH
        ret

