;****** Servo Tester 2
;       InputStat include file
;       Input statistics menu
;
;	Copyright (c) 2004-11, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2

.equ    InputStat_RefreshTimeStaticInput= 3     ;3*100ms = 0.3 s
.equ    InputStat_DifferenceForUpdate   = 50    ;Minimum difference to force display update outside timer refresh

.dseg
InputStatMin:           .byte 2
InputStatMax:           .byte 2
InputStatDropouts:      .byte 2         ;Incremented from an interrupt!
InputStatDropRel:       .byte 2         ;Copy of Dropouts, but can be resized at runtime
InputStatTotal:         .byte 2         ;Incremented from an interrupt!
InputStatSpikesLow:     .byte 2         ;Incremented from an interrupt!
InputStatSpikesHigh:    .byte 2         ;Incremented from an interrupt!
InputStatSpikesLowRel:  .byte 2         ;Copy of SpikesLow, but can be resized at runtime
InputStatSpikesHighRel: .byte 2         ;Copy of SpikesHigh, but can be resized at runtime
InputStatFrameLen:      .byte 3
InputStatRefreshCount:  .byte 1



.cseg


InputStat_ServiceTable:
        jmp    InputStat_Init
        jmp    InputStat_DisplayMenu
        jmp    InputStat_TakeControl
        jmp    InputStat_Left
        jmp    InputStat_Right
        jmp    InputStat_Push
        jmp    InputStat_UpdateDisplay
        jmp    InputStat_ICP
        jmp    InputStat_TimerTick
        jmp    InputStat_ADComplete
        jmp    InputStat_UpdateCursor
        jmp    InputStat_OutputSent
        jmp    InputStat_TachoUpdate
        jmp    InputStat_Hold


InputStat_Init:
        rcall   InputStat_ResetCounters_w12
        ldi     w1,InputStat_RefreshTimeStaticInput
        sts     InputStatRefreshCount,w1
        ret


InputStat_ResetCounters_w12:
        ser     w1
        sts     InputStatMin,w1
        sts     InputStatMin+1,w1
        clr     w1
        sts     InputStatMax,w1
        sts     InputStatMax+1,w1
        in      w2,SREG
        cli
        sts     InputStatDropouts,w1
        sts     InputStatDropouts+1,w1
        sts     InputStatDropRel,w1
        sts     InputStatDropRel+1,w1
        sts     InputStatTotal,w1
        sts     InputStatTotal+1,w1
        sts     InputStatSpikesLow,w1
        sts     InputStatSpikesLow+1,w1
        sts     InputStatSpikesHigh,w1
        sts     InputStatSpikesHigh+1,w1
        sts     InputStatSpikesLowRel,w1
        sts     InputStatSpikesLowRel+1,w1
        sts     InputStatSpikesHighRel,w1
        sts     InputStatSpikesHighRel+1,w1
        out     SREG,w2
        ret


InputStat_UpdateInputs_All:     ;New input value in X
        mov     w1,XL
        or      w1,XH
        breq    ISUI_Exit       ;Ignore simulated 0-input
        lds     YL,InputStatMin
        lds     YH,InputStatMin+1
        cp      XL,YL
        cpc     XH,YH
        brsh    ISUI_NotLow
        sts     InputStatMin,XL
        sts     InputStatMin+1,XH
ISUI_NotLow:
        lds     YL,InputStatMax
        lds     YH,InputStatMax+1
        cp      YL,XL
        cpc     YH,XH
        brsh    ISUI_NotHigh
        sts     InputStatMax,XL
        sts     InputStatMax+1,XH
ISUI_NotHigh:
        rcall   InputStat_IncrementTotal_w1X
ISUI_Exit:
        ret


InputStat_IncrementTotal_w1X:
        in      w1,SREG
        cli
        lds     XL,InputStatTotal
        lds     XH,InputStatTotal+1
        adiw    XH:XL,1
        sts     InputStatTotal,XL
        sts     InputStatTotal+1,XH
        brcc    ISIT_NotWrap
        ldi     XL,low(0x8000)
        ldi     XH,high(0x8000)
        sts     InputStatTotal,XL
        sts     InputStatTotal+1,XH
        lds     XL,InputStatDropRel
        lds     XH,InputStatDropRel+1
        lsr     XH
        ror     XL
        sts     InputStatDropRel,XL
        sts     InputStatDropRel+1,XH
        lds     XL,InputStatSpikesLowRel
        lds     XH,InputStatSpikesLowRel+1
        lsr     XH
        ror     XL
        sts     InputStatSpikesLowRel,XL
        sts     InputStatSpikesLowRel+1,XH
        lds     XL,InputStatSpikesHighRel
        lds     XH,InputStatSpikesHighRel+1
        lsr     XH
        ror     XL
        sts     InputStatSpikesHighRel,XL
        sts     InputStatSpikesHighRel+1,XH
ISIT_NotWrap:
        out     SREG,w1
        ret


InputStat_LowSpikeDetected_w1X:
        in      w1,SREG
        push    w1
        cli
        lds     XL,InputStatSpikesLow
        lds     XH,InputStatSpikesLow+1
        adiw    XH:XL,1
        brcs    ISLSD_Limit1
        sts     InputStatSpikesLow,XL
        sts     InputStatSpikesLow+1,XH
ISLSD_Limit1:
        lds     XL,InputStatSpikesLowRel
        lds     XH,InputStatSpikesLowRel+1
        adiw    XH:XL,1
        brcs    ISLSD_Limit2
        sts     InputStatSpikesLowRel,XL
        sts     InputStatSpikesLowRel+1,XH
ISLSD_Limit2:
        rcall   InputStat_IncrementTotal_w1X
        pop     w1
        out     SREG,w1
        ret


InputStat_HighSpikeDetected_w1X:
        in      w1,SREG
        push    w1
        cli
        lds     XL,InputStatSpikesHigh
        lds     XH,InputStatSpikesHigh+1
        adiw    XH:XL,1
        brcs    ISHSD_Limit1
        sts     InputStatSpikesHigh,XL
        sts     InputStatSpikesHigh+1,XH
ISHSD_Limit1:
        lds     XL,InputStatSpikesHighRel
        lds     XH,InputStatSpikesHighRel+1
        adiw    XH:XL,1
        brcs    ISHSD_Limit2
        sts     InputStatSpikesHighRel,XL
        sts     InputStatSpikesHighRel+1,XH
ISHSD_Limit2:
        rcall   InputStat_IncrementTotal_w1X
        pop     w1
        out     SREG,w1
        ret


InputStat_DropoutDetected_w1X:
        in      w1,SREG
        push    w1
        cli
        lds     XL,InputStatDropouts
        lds     XH,InputStatDropouts+1
        adiw    XH:XL,1
        brcs    ISDD_Limit1
        sts     InputStatDropouts,XL
        sts     InputStatDropouts+1,XH
ISDD_Limit1:
        lds     XL,InputStatDropRel
        lds     XH,InputStatDropRel+1
        adiw    XH:XL,1
        brcs    ISDD_Limit2
        sts     InputStatDropRel,XL
        sts     InputStatDropRel+1,XH
ISDD_Limit2:
        rcall   InputStat_IncrementTotal_w1X
        pop     w1
        out     SREG,w1
        ret



InputStat_DisplayMenu:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*InputStat_DMenu1)
        ldi     ZH,high(2*InputStat_DMenu1)
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*InputStat_DMenu2)
        ldi     ZH,high(2*InputStat_DMenu2)
        call    LCD_WriteString_Z
        ret

InputStat_DMenu1:       .db "  Input Stat    ",0,0
InputStat_DMenu2:       .db "                ",0,0


InputStat_TakeControl:
        ldi     c1,1
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


InputStat_Left:
        tst     c1
        breq    ISL_Exit
        dec     c1
ISL_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


InputStat_Right:
        cpi     c1,7
        brsh    ISR_Exit
        inc     c1
ISR_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ret


InputStat_Push:
        tst     c1
        brne    ISP_NotMenuExit
        ldi     w1,EVENT_ReleaseControl
        call    EventQ_PutEventInFront
        rjmp    ISP_Exit
ISP_NotMenuExit:
        rcall   InputStat_ResetCounters_w12
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
ISP_Exit:
        ret


InputStat_UpdateDisplay:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*InputStat_MenuL1)
        ldi     ZH,high(2*InputStat_MenuL1)
        ldi     w1,18                           ;18 bytes between text lines
        mul     w1,c1
        add     ZL,r0
        adc     ZH,r1
        call    LCD_WriteString_Z               ;Write line 1
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        tst     c1
        brne    ISUD_Not0
        ldi     ZL,low(2*InputStat_Menu02)
        ldi     ZH,high(2*InputStat_Menu02)
        call    LCD_WriteString_Z
        rjmp    ISUD_Exit
ISUD_Not0:
        cpi     c1,1
        brne    ISUD_Not1
        lds     w1,InputStatMin
        lds     w2,InputStatMin+1
        and     w1,w2
        cpi     w1,0xff
        brne    ISUD1_ValidPulse
        rjmp    ISUD_NoValidInput
ISUD1_ValidPulse:
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        lds     XL,InputStatMin
        lds     XH,InputStatMin+1
        call    Util_WritePulseWidth_XY
        ldi     ZL,low(2*InputStat_Menu12b)
        ldi     ZH,high(2*InputStat_Menu12b)
        call    LCD_WriteString_Z
        rjmp    ISUD_Exit
ISUD_Not1:
        cpi     c1,2
        brne    ISUD_Not2
        lds     w1,InputStatMin
        lds     w2,InputStatMin+1
        and     w1,w2
        cpi     w1,0xff
        brne    ISUD2_ValidPulse
        rjmp    ISUD_NoValidInput
ISUD2_ValidPulse:
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        lds     XL,InputStatMax
        lds     XH,InputStatMax+1
        call    Util_WritePulseWidth_XY
        ldi     ZL,low(2*InputStat_Menu12b)
        ldi     ZH,high(2*InputStat_Menu12b)
        call    LCD_WriteString_Z
        rjmp    ISUD_Exit
ISUD_Not2:
        cpi     c1,3
        brne    ISUD_Not3
        lds     w1,InputStatMin
        lds     w2,InputStatMin+1
        and     w1,w2
        cpi     w1,0xff
        brne    ISUD3_ValidPulse
        rjmp    ISUD_NoValidInput
ISUD3_ValidPulse:
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        lds     XL,InputStatMax
        lds     XH,InputStatMax+1
        lds     YL,InputStatMin
        lds     YH,InputStatMin+1
        sub     XL,YL
        sbc     XH,YH
        brcc    ISUD3_NotBelow0
        clr     XL
        clr     XH
ISUD3_NotBelow0:
        call    Util_WritePulseWidth_XY
        ldi     ZL,low(2*InputStat_Menu12b)
        ldi     ZH,high(2*InputStat_Menu12b)
        call    LCD_WriteString_Z
        rjmp    ISUD_Exit
ISUD_Not3:
        cpi     c1,4
        brne    ISUD_Not4
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        rcall   InputStat_DisplayMenu4
        rjmp    ISUD_Exit
ISUD_Not4:
        cpi     c1,5
        brne    ISUD_Not5
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        in      w1,SREG
        cli
        lds     XL,InputStatDropouts
        lds     XH,InputStatDropouts+1
        out     SREG,w1
        call    Util_WriteU16_5d_XY
        ldi     w1,' '
        call    LCD_SendData
        in      w1,SREG
        cli
        lds     XL,InputStatDropRel
        lds     XH,InputStatDropRel+1
        lds     ZL,InputStatTotal
        lds     ZH,InputStatTotal+1
        out     SREG,w1
        rcall   InputStat_WritePercentage
        rjmp    ISUD_Exit
ISUD_Not5:
        cpi     c1,6
        brne    ISUD_Not6
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        in      w1,SREG
        cli
        lds     XL,InputStatSpikesLow
        lds     XH,InputStatSpikesLow+1
        out     SREG,w1
        call    Util_WriteU16_5d_XY
        ldi     w1,' '
        call    LCD_SendData
        in      w1,SREG
        cli
        lds     XL,InputStatSpikesLowRel
        lds     XH,InputStatSpikesLowRel+1
        lds     ZL,InputStatTotal
        lds     ZH,InputStatTotal+1
        out     SREG,w1
        rcall   InputStat_WritePercentage
        rjmp    ISUD_Exit
ISUD_Not6:
;        cpi     c1,7                           ;7 is last possible menu
;        brne    ISUD_Not7
        ldi     ZL,low(2*InputStat_Menu12a)
        ldi     ZH,high(2*InputStat_Menu12a)
        call    LCD_WriteString_Z
        in      w1,SREG
        cli
        lds     XL,InputStatSpikesHigh
        lds     XH,InputStatSpikesHigh+1
        out     SREG,w1
        call    Util_WriteU16_5d_XY
        ldi     w1,' '
        call    LCD_SendData
        in      w1,SREG
        cli
        lds     XL,InputStatSpikesHighRel
        lds     XH,InputStatSpikesHighRel+1
        lds     ZL,InputStatTotal
        lds     ZH,InputStatTotal+1
        out     SREG,w1
        rcall   InputStat_WritePercentage
        rjmp    ISUD_Exit
;ISUD_Not7:
ISUD_NoValidInput:
        ldi     ZL,low(2*InputStat_Menu12Err)
        ldi     ZH,high(2*InputStat_Menu12Err)
        call    LCD_WriteString_Z
;        rjmp    ISUD_Exit
ISUD_Exit:
        ret

;Keep these lines together (and always 18 chars long)
InputStat_MenuL1:
        .db LCD_CharRightArrow," Push to exit  ",0,0
        .db LCD_CharRightArrow," Input minimum ",0,0
        .db LCD_CharRightArrow," Input maximum ",0,0
        .db LCD_CharRightArrow," Input jitter  ",0,0
        .db LCD_CharRightArrow," Input framelen",0,0
        .db LCD_CharRightArrow," Input dropouts",0,0
        .db LCD_CharRightArrow," Input too low ",0,0
        .db "  Input too high",0,0

InputStat_Menu02:       .db "   Input Stat   ",0,0
InputStat_Menu12a:      .db LCD_CharLeftArrow," ",0,0
InputStat_Menu12b:      .db "     ",0
InputStat_Menu12Err:    .db LCD_CharLeftArrow," No valid input",0,0
InputStat_Menu4ms:      .db " ms",0
InputStat_Menu4Hz:      .db " Hz",0


InputStat_WritePercentage:                      ;Subfunction to UpdateDisplay
        mov     w1,ZL
        or      w1,ZH                           ;Check for divide by zero
        brne    ISWP_NotZero
        clr     XL
        clr     XH
        clr     u1
        clr     u2
        rjmp    ISWP_WriteValue
ISWP_NotZero:
        ldi     YL,low(10000)
        ldi     YH,high(10000)
        call    mul16x16_32_w                   ;X * 10000
        movw    XH:XL,w2:w1
        movw    YH:YL,w4:w3
        call    div32_16u_uw1XYZ                ;Result / Z (Z=total)
        ldi     ZL,low(100)
        ldi     ZH,high(100)
        call    div32_16u_uw1XYZ                ;Result / 100
ISWP_WriteValue:
        call    Util_WriteU8_3d_XY
        ldi     w1,'.'
        call    LCD_SendData
        movw    XH:XL,u2:u1
        call    Util_WriteU8_2d00_XY
        ldi     w1,' '
        call    LCD_SendData
        ldi     w1,'%'
        call    LCD_SendData
        ret


InputStat_DisplayMenu4:                         ;Subfunction to menu 4 in UpdateDisplay
        lds     XL,InputStatFrameLen
        lds     XH,InputStatFrameLen+1
        lds     YL,InputStatFrameLen+2
        clr     YH
        mov     w1,XL
        or      w1,XH
        or      w1,YL
        breq    ISDM4_InvalidData
        ldi     ZL,low(100)
        ldi     ZH,high(100)
        call    div32_16u_uw1XYZ
        ldi     w1,low(10000)
        ldi     w2,high(10000)
        ldi     w3,byte3(10000)
        ldi     w4,byte4(10000)
        cp      XL,w1
        cpc     XH,w2
        cpc     YL,w3
        cpc     YH,w4
        brlo    ISDM4_Below10k
        ldi     ZL,low(100)             ; >= 100 ms
        ldi     ZH,high(100)
        call    div32_16u_uw1XYZ
        ldi     w1,low(1000)
        ldi     w2,high(1000)
        cp      XL,w1
        cpc     XH,w2
        brsh    ISDM4_InvalidData
        ldi     w2,3                    ; 3 digits, no decimal point
        ldi     w6,1                    ; Flag for x.xx Hz display later
        rjmp    ISDM4_DisplayFrameLength
ISDM4_Below10k:
        ldi     w1,low(1000)
        ldi     w2,high(1000)
        cp      XL,w1
        cpc     XH,w2
        brlo    ISDM4_Below1k
        ldi     ZL,low(10)              ; >= 10 ms
        ldi     ZH,high(10)
        call    div32_16u_uw1XYZ
        ldi     w2,2                    ; 2 digits before decimal point
        clr     w6                      ; Flag for xxx Hz display later
        rjmp    ISDM4_DisplayFrameLength
ISDM4_Below1k:
        ldi     w2,1                    ; 1 digit before decimal point
        clr     w6                      ; Flag for xxx Hz display later
        rjmp    ISDM4_DisplayFrameLength
ISDM4_InvalidData:
        ldi     ZL,low(2*InputStat_Menu12Err+2)
        ldi     ZH,high(2*InputStat_Menu12Err+2)
        call    LCD_WriteString_Z
        rjmp    ISDM4_Exit
ISDM4_DisplayFrameLength:
        clt                             ; Blank leading zeros
        call    Util_Write3digit_w12XY
        ldi     ZL,low(2*InputStat_Menu4ms)
        ldi     ZH,high(2*InputStat_Menu4ms)
        call    LCD_WriteString_Z
ISDM4_CalculateFrequency:
        lds     XL,InputStatFrameLen            ; Re-read original time
        lds     XH,InputStatFrameLen+1
        lds     YL,InputStatFrameLen+2
        clr     YH
        ldi     ZL,low(200)                     ; Divide by 200
        ldi     ZH,high(200)
        call    div32_16u_uw1XYZ
        movw    ZH:ZL,XH:XL                     ; Result as new divisor (which is max 50000)
        tst     w6                              ; See what kind of frequency we should display
        brne    ISDM4_DisplayLowHz
        ldi     XL,low(100000)
        ldi     XH,high(100000)
        ldi     YL,byte3(100000)
        ldi     YH,byte4(100000)
        call    div32_16u_uw1XYZ
        adiw    XH:XL,1
        lsr     XH
        ror     XL
        ldi     w2,4                    ; 4 digits, no decimal point
        clt                             ; Blank leading zeros
        call    Util_Write4digit_w12XY
        ldi     ZL,low(2*InputStat_Menu4Hz)
        ldi     ZH,high(2*InputStat_Menu4Hz)
        call    LCD_WriteString_Z
        rjmp    ISDM4_Exit
ISDM4_DisplayLowHz:
        ldi     XL,low(10000000)
        ldi     XH,high(10000000)
        ldi     YL,byte3(10000000)
        ldi     YH,byte4(10000000)
        call    div32_16u_uw1XYZ
        adiw    XH:XL,1
        lsr     XH
        ror     XL
        ldi     w2,2                    ; 2 digits before decimal point
        clt                             ; Blank leading zeros
        call    Util_Write4digit_w12XY
        ldi     ZL,low(2*InputStat_Menu4Hz)
        ldi     ZH,high(2*InputStat_Menu4Hz)
        call    LCD_WriteString_Z
ISDM4_Exit:
        ret


InputStat_ICP:
        cpi     c1,4
        breq    ISICP_Exit                      ;Update display unless we are in menu 4
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
ISICP_Exit:
        ret


InputStat_TimerTick:
        lds     w1,InputStatRefreshCount
        dec     w1
        brne    ISTT_Exit
        ldi     w1,EVENT_Display                ;Request display update (mainly for menu 4)
        call    EventQ_PutEventIfNew
        ldi     w1,InputStat_RefreshTimeStaticInput
ISTT_Exit:
        sts     InputStatRefreshCount,w1
        ret


InputStat_ADComplete:
        ret


InputStat_UpdateCursor:
        ret


InputStat_OutputSent:
        ret


InputStat_TachoUpdate:
        ret


InputStat_Hold:
        ret


