;****** Servo Tester 2
;       LCD include file
;       LCD driver
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    LCD_PORT        =PORTB
.equ    LCD_PIN         =PINB
.equ    LCD_DDR         =DDRB
.equ    LCD_RS          =PB1    ;Portbit
.equ    LCD_RW          =PB2    ;Portbit
.equ    LCD_E           =PB3    ;Portbit
.equ    LCD_PORTMASK    =0xFE   ;Bit 0 not in use for LCD


.cseg

LCD_SendHighNibble_w12:         ;Nibble in w1
        andi    w1,0b11110000
        sbi     LCD_PORT,LCD_E          ;Set E
        in      w2,LCD_PORT
        andi    w2,0b00001111
        or      w2,w1
        out     LCD_PORT,w2
        cbi     LCD_PORT,LCD_E          ;Clear E
        ret


LCD_LocalSend:
        push    w2
        push    w1
        rcall   LCD_SendHighNibble_w12
        ldi     w1,1
        call    Util_Delayus            ;1 us delay
        pop     w1
        push    w1
        swap    w1
        rcall   LCD_SendHighNibble_w12
        ldi     w1,43
        call    Util_Delayus            ;Delay 43 us
        pop     w1
        pop     w2
        ret


LCD_ReadHighNibble_w1:
        sbi     LCD_PORT,LCD_E          ;Set E
        nop                             ;500 ns delay (@ 10 MHz xtal)
        nop
        nop
        nop
        nop
        in      w1,LCD_PIN
        cbi     LCD_PORT,LCD_E
        andi    w1,0b11110000
        ret


LCD_LocalRead_w1:
        push    w2
        in      w2,LCD_PORT
        ori     w2,0b11110000
        out     LCD_PORT,w2             ;Data=1111 (enabling pullups when reconfiguring for input)
        in      w2,LCD_DDR
        andi    w2,0b00001111
        out     LCD_DDR,w2
        rcall   LCD_ReadHighNibble_w1
        mov     w2,w1
        nop                             ;500 ns delay (@ 10 MHz xtal)
        nop
        nop
        nop
        nop
        rcall   LCD_ReadHighNibble_w1
        swap    w1
        or      w2,w1
        ldi     w1,43
        call    Util_Delayus            ;Delay 43 us
        mov     w1,w2
        in      w2,LCD_DDR
        ori     w2,0b11110000
        out     LCD_DDR,w2
        pop     w2
        ret


LCD_SendCommand:                        ;Command in w1
        cbi     LCD_PORT,LCD_RS         ;Clear RS
        cbi     LCD_PORT,LCD_RW         ;Clear R/W (Write)
        rjmp    LCD_LocalSend


LCD_SendData:                           ;Data in w1
        sbi     LCD_PORT,LCD_RS         ;Set RS
        cbi     LCD_PORT,LCD_RW         ;Clear R/W (Write)
        rjmp    LCD_LocalSend


LCD_ReadStatus_w1:                      ;Status in w1
        cbi     LCD_PORT,LCD_RS         ;Clear RS
        sbi     LCD_PORT,LCD_RW         ;Set R/W (Read)
        rjmp    LCD_LocalRead_w1


LCD_ReadData_w1:                        ;Data in w1
        sbi     LCD_PORT,LCD_RS         ;Set RS
        sbi     LCD_PORT,LCD_RW         ;Set R/W (Read)
        rjmp    LCD_LocalRead_w1


LCD_Init:
        push    w1
        push    w2
        in      w1,LCD_DDR
        ori     w1,LCD_PORTMASK
        out     LCD_DDR,w1
        ldi     XL,15
        clr     XH
        call    Util_Delayms_X          ;Delay 15 ms
        ldi     w1,0x30
        rcall   LCD_SendHighNibble_w12
        ldi     XL,5
        clr     XH
        call    Util_Delayms_X          ;Delay 5 ms
        ldi     w1,0x30
        rcall   LCD_SendHighNibble_w12
        ldi     w1,100
        call    Util_Delayus            ;Delay 100 us
        ldi     w1,0x30
        rcall   LCD_SendHighNibble_w12
        ldi     w1,40
        call    Util_Delayus            ;Delay 40 us
        ldi     w1,0x20
        rcall   LCD_SendHighNibble_w12
        ldi     w1,40
        call    Util_Delayus            ;Delay 40 us
        ldi     w1,0x28                 ;Init1: 4bit / 2 line / 5x7 dot format
        rcall   LCD_SendCommand
        ldi     w1,0x06                 ;Init2: Entry mode set / Increment / No shift
        rcall   LCD_SendCommand
        ldi     w1,0x0C                 ;Init3: Display ctrl. / Disp ON / Cursor OFF / Blink OFF
        rcall   LCD_SendCommand
        ldi     w1,0x01                 ;Init4: Display Clear
        rcall   LCD_SendCommand
        ldi     XL,2
        clr     XH
        call    Util_Delayms_X          ;Delay 2 ms
        rcall   LCD_SetCustomCharset1_w1Z       ;Init5: Load CGRAM
        ldi     w1,0x80                 ;Init6: DD RAM Address Set / 1st char
        rcall   LCD_SendCommand
        pop     w2
        pop     w1
        ret


LCD_SetCustomCharset1_w1Z:
        ldi     ZL,low(2*LCD_CustomChars1)
        ldi     ZH,high(2*LCD_CustomChars1)
        rjmp    LCDSCC_Common
LCD_SetCustomCharset2_w1Z:
        ldi     ZL,low(2*LCD_CustomChars2)
        ldi     ZH,high(2*LCD_CustomChars2)
LCDSCC_Common:
        ldi     w1,0x40                 
        rcall   LCD_SendCommand
LCDSCC_Loop:
        lpm     w1,Z+
        rcall   LCD_SendData
        sbrs    w1,7
        rjmp    LCDSCC_Loop
        ret

LCD_CustomChars1:                       ;End table with 0x80
        .db     0x04,0x04,0x1f,0x04,0x04,0x00,0x1f,0x00         ;+/-
        .db     0x00,0x01,0x02,0x02,0x04,0x14,0x08,0x00         ;Select
        .db     0x04,0x0e,0x15,0x04,0x04,0x04,0x04,0x00         ;Arrow up
        .db     0x04,0x04,0x04,0x04,0x15,0x0e,0x04,0x00         ;Arrow down
        .db     0x00,0x04,0x02,0x1f,0x02,0x04,0x00,0x00         ;Arrow right
        .db     0x00,0x04,0x08,0x1f,0x08,0x04,0x00,0x00         ;Arrow left
        .db     0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x80         ;Block

.equ    LCD_CharAddSub          =8      ;0 can't be used. Char 8 is a duplicate of char 0
.equ    LCD_CharSelect          =1
.equ    LCD_CharUpArrow         =2
.equ    LCD_CharDownArrow       =3
.equ    LCD_CharRightArrow      =4
.equ    LCD_CharLeftArrow       =5
.equ    LCD_CharBlock           =6

LCD_CustomChars2:                       ;End table with 0x80
        .db     0x04,0x04,0x1f,0x04,0x04,0x00,0x1f,0x00         ;+/-
        .db     0x00,0x01,0x02,0x02,0x04,0x14,0x08,0x00         ;Select
        .db     0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00         ;TopLine
        .db     0x00,0x00,0x00,0x00,0x00,0x00,0x1f,0x80         ;BottomLine

.equ    LCD_CharTopLine         =2
.equ    LCD_CharBottomLine      =3
.equ    LCD_FirstUserChar       =4


LCD_WriteString_Z:
        push    w1
LCDWS_Loop:
        lpm     w1,Z+
        tst     w1
        breq    LCDWS_Exit
        rcall   LCD_SendData
        rjmp    LCDWS_Loop
LCDWS_Exit:
        pop     w1
        ret


;******************************************
;* FUNCTION
;*      LCD_GotoXY
;* PARAMETERS
;*      w1: X[0..15], w2: Y[0..1]
;* DESCRIPTION
;*      Sets DD RAM address
;******************************************
LCD_GotoXY:
        push    w1
        push    w2
        andi    w1,0x0f
        andi    w2,0x01
        swap    w2
        lsl     w2
        lsl     w2
        add     w1,w2
        ori     w1,0x80
        rcall   LCD_SendCommand
        pop     w2
        pop     w1
        ret


LCD_CursorOn:
        push    w1
        ldi     w1,0x0f                 ;Display ctrl. / Disp ON / Cursor ON / Blink ON
        rcall   LCD_SendCommand
        pop     w1
        ret


LCD_CursorOff:
        push    w1
        ldi     w1,0x0c                 ;Display ctrl. / Disp ON / Cursor OFF / Blink OFF
        rcall   LCD_SendCommand
        pop     w1
        ret


LCD_SetCustomChar:              ; Set a custom char w1 to pattern in u1-u7
        push    w2
        push    w1
        push    w1
        rcall   LCD_ReadStatus_w1       ; Read cursor position
        mov     w2,w1
        pop     w1
        lsl     w1
        lsl     w1
        lsl     w1
        ori     w1,0x40
        rcall   LCD_SendCommand
        mov     w1,u1
        rcall   LCD_SendData
        mov     w1,u2
        rcall   LCD_SendData
        mov     w1,u3
        rcall   LCD_SendData
        mov     w1,u4
        rcall   LCD_SendData
        mov     w1,u5
        rcall   LCD_SendData
        mov     w1,u6
        rcall   LCD_SendData
        mov     w1,u7
        rcall   LCD_SendData
        clr     w1
        rcall   LCD_SendData
        mov     w1,w2
        ori     w1,0x80
        rcall   LCD_SendCommand         ; Restore cursor position
        pop     w1
        pop     w2
        ret

