;****** Servo Tester 2
;       Pulse include file
;       Servo pulse driver
;
;       Copyright (c) 2004-11, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    Pulse_AliveTimerReload  = 10    ;10*10ms = 0.1 s

.dseg

PulseInput:      .byte 2                 ;Latest read pulsewidth

PulseICP:        .byte 2                 ;Internal work mem
PulseState:      .byte 1
PulseAliveTimer: .byte 1
PulseInputCount: .byte 1
PulseInputMax:   .byte 1

PulseOutLocalA:  .byte 2
PulseOutLocalB:  .byte 2

PulseFrameB3:    .byte 1
PulseFrameLast:  .byte 3
PulseFrameLen:   .byte 3                ;Latest time between two frames



.cseg

Pulse_InputCapture_int:                 ; Timer1 Input Capture Handler
        push    w1
        in      w1,SREG
        push    w1
        push    w2
        push    XL
        push    XH
        in      w1,TCCR1B               ;Reverse edge trigger on Input Capture
        ldi     w2,(1<<ICES1)           ;to capture both edges
        eor     w1,w2
        out     TCCR1B,w1
        sbrc    w1,ICES1
        rjmp    PIC_PulseEnd
        in      w1,ICR1L                ;Positive edge. Get captured value
        in      w2,ICR1H
        sts     PulseICP,w1             ;And save it for later use
        sts     PulseICP+1,w2
        lds     XL,PulseFrameB3         ;Read 3rd byte to determine frame length
        sbrc    w2,7
        rjmp    PIC_3rdByteCalculated
        in      XH,TIFR
        sbrc    XH,TOV1
        inc     XL                      ;Wrap around detected. Increment 3rd byte
PIC_3rdByteCalculated:                  ;Current time is in XL:w2:w1
        lds     XH,PulseFrameLast       ;Use XH as temporary register
        sts     PulseFrameLast,w1
        sub     w1,XH
        lds     XH,PulseFrameLast+1
        sts     PulseFrameLast+1,w2
        sbc     w2,XH
        lds     XH,PulseFrameLast+2
        sts     PulseFrameLast+2,XL
        sbc     XL,XH
        sts     PulseFrameLen,w1        ;Time difference to last frame is in XL:w2:w1
        sts     PulseFrameLen+1,w2
        sts     PulseFrameLen+2,XL
        ;
        rjmp    PIC_Exit
PIC_PulseEnd:
        in      w1,ICR1L                ;Negative edge. Get captured value
        in      w2,ICR1H
        lds     XL,PulseICP
        lds     XH,PulseICP+1
        sub     w1,XL                   ;Find difference from positive edge
        sbc     w2,XH                   ;to get pulsewidth
        call    Config_GetMinPW_X
        cp      w1,XL                   ;Check boundaries
        cpc     w2,XH
        brlo    PIC_RangeErrorLow
        call    Config_GetMaxPW_X
        cp      XL,w1
        cpc     XH,w2
        brlo    PIC_RangeErrorHigh
        sts     PulseInput,w1
        sts     PulseInput+1,w2
        ldi     w1,EVENT_ICP            ;Announce new input value
        call    EventQ_PutEvent
        ldi     w1,Pulse_AliveTimerReload
        sts     PulseAliveTimer,w1      ;Reset input alive watchdog
        lds     w1,PulseInputCount
        inc     w1
        sts     PulseInputCount,w1
        rjmp    PIC_Exit
PIC_RangeErrorLow:
        call    InputStat_LowSpikeDetected_w1X
        rjmp    PIC_Exit
PIC_RangeErrorHigh:
        call    InputStat_HighSpikeDetected_w1X
;        rjmp    PIC_Exit
PIC_Exit:
        pop     XH
        pop     XL
        pop     w2
        pop     w1
        out     SREG,w1
        pop     w1
        ret


Pulse_10ms_int_w12:
        lds     w2,PulseAliveTimer
        dec     w2
        brne    P10MSIRQ_NotTimeout
        ldi     w2,Pulse_AliveTimerReload
        clr     w1
        sts     PulseInput,w1           ;Clear input value
        sts     PulseInput+1,w1
        sts     PulseFrameLen,w1
        sts     PulseFrameLen+1,w1
        sts     PulseFrameLen+2,w1
        ldi     w1,EVENT_ICP            ;Announce new (fake) input value
        call    EventQ_PutEvent
P10MSIRQ_NotTimeout:
        sts     PulseAliveTimer,w2
        ret


Pulse_100ms_int_w12:
        lds     w1,PulseInputCount
        lds     w2,PulseInputMax
        cp      w2,w1
        brlo    P100MSIRQ_HigherInputHit
        inc     w1
        cp      w1,w2
        brsh    P100MSIRQ_Exit          ;Within expected boundaries -> Exit
        push    XL
        push    XH
        call    InputStat_DropoutDetected_w1X
        pop     XH
        pop     XL
        clr     w1
P100MSIRQ_HigherInputHit:
        sts     PulseInputMax,w1
P100MSIRQ_Exit:
        clr     w1
        sts     PulseInputCount,w1
        ret


Pulse_Overflow_int:                     ;Timer1 Overflow Handler
        push    w1
        in      w1,SREG
        push    w1
        push    w2
        lds     w1,PulseFrameB3         ;Increment 3rd byte for frame time measurement
        inc     w1
        sts     PulseFrameB3,w1
        lds     w2,PulseState
        cpi     w2,0
        breq    PO_State0               ;Find current pulsewidth generator state
        cpi     w2,1
        breq    PO_State1
        cpi     w2,2
        breq    PO_State2
        clr     w2                      ;Error. Unknown state. Reinit
        rjmp    PO_Exit
PO_State0:                              ;State 0: Preparing for generation of new pulse
        inc     w2                      ;At next TC1 overflow, the positive edge will be generated by hardware
        clr     w1
        out     OCR1AH,w1               ;Set compare value A to 0
        out     OCR1AL,w1
        out     OCR1BH,w1               ;Set compare value B to 0
        out     OCR1BL,w1
        push    w2
        push    w3
        ldi     w3,(1<<COM1A1)+(1<<COM1B1)
        lds     w1,RouterOutputA
        call    Router_GetPulse_w12
        sts     PulseOutLocalA,w1
        sts     PulseOutLocalA+1,w2
        or      w1,w2
        breq    PO_DisableOutputA
        sbr     w3,(1<<COM1A0)
PO_DisableOutputA:
        lds     w1,RouterOutputB
        call    Router_GetPulse_w12
        sts     PulseOutLocalB,w1
        sts     PulseOutLocalB+1,w2
        or      w1,w2
        breq    PO_DisableOutputB
        sbr     w3,(1<<COM1B0)
PO_DisableOutputB:
        out     TCCR1A,w3               ;Set OC to 1 at compare match if not disabled
        pop     w3
        pop     w2
        rjmp    PO_Exit
PO_State1:                              ;State 1: New pulse just started. Set up hardware for negative edge gen.
        inc     w2
        lds     w1,PulseOutLocalA+1
        out     OCR1AH,w1
        lds     w1,PulseOutLocalA
        out     OCR1AL,w1
        lds     w1,PulseOutLocalB+1
        out     OCR1BH,w1
        lds     w1,PulseOutLocalB
        out     OCR1BL,w1
        ldi     w1,(1<<COM1A1)+(1<<COM1B1)
        out     TCCR1A,w1               ;Set OC to 0 at compare match
        ldi     w1,EVENT_OutputSent     ;Announce use of new output values
        call    EventQ_PutEventInFront
        rjmp    PO_Exit
PO_State2:                              ;State 2: Delay before next pulse
        clr     w2
PO_Exit:
        sts     PulseState,w2
        pop     w2
        pop     w1
        out     SREG,w1
        pop     w1
        ret



Pulse_Init:
        clr     w1
        sts     PulseState,w1
        sts     PulseInput,w1
        sts     PulseInput+1,w1
        sts     PulseInputCount,w1
        sts     PulseInputMax,w1
        ldi     w1,Pulse_AliveTimerReload
        sts     PulseAliveTimer,w1
        ldi     w1,(1<<COM1A1)+(1<<COM1B1)
        out     TCCR1A,w1               ;OC1 A and B output clear on compare match
        sbr     w1,(1<<FOC1A)+(1<<FOC1B)
        out     TCCR1A,w1               ;Clear OC1 pins
        ldi     w1,(1<<ICNC1)+(1<<ICES1)+(1<<CS10)
        out     TCCR1B,w1               ;Input capture noise canceller, capture at rising edge, count at CK rate
        in      w1,TIMSK
        sbr     w1,(1<<TICIE1)+(1<<TOIE1)
        cbr     w1,(1<<OCIE1A)+(1<<OCIE1B)
        out     TIMSK,w1                ;Interrupt at TC1 overflow and input capture
        in      w1,DDRD
        sbr     w1,(1<<PD4)+(1<<PD5)
        out     DDRD,w1                 ;OC1 pins output
        sbi     PORTD,PD6               ;Enable pullup at ICP
        ret

