;****** Servo Tester 2
;       PWIO include file
;       Pulse Width I/O generic menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.dseg
PWIOAddress:            .byte 2
PWIOBarAddressLow:      .byte 2         ; Adr. of low limit for analogue mode
PWIOBarAddressHigh:     .byte 2         ; Adr. of high limit for analogue mode
PWIORouterAdr:          .byte 1
PWIOText1Ptr:           .byte 2
PWIOExtendedCallback:   .byte 2
PWIOMinPW:              .byte 2
PWIOMaxPW:              .byte 2
PWIOInControl:          .byte 1


.equ PWIOC2EditMode     = 0
.equ PWIOC2CursorReverse= 1
.equ PWIOC2BarMode      = 2

.equ PWIOC3EditPW       = 0
.equ PWIOC3DisplayAB    = 1
.equ PWIOC3ExtendedMenu = 2
.equ PWIOC3AllowBarMode = 3

.equ PWIO_EXTENDLEFT  = 0
.equ PWIO_EXTENDRIGHT = 1


.cseg

PWIO_Init:                                      ;Called at power-up
        clr     w1
        sts     PWIOInControl,w1
        ret
        

PWIO_TakeControl:                               ;Called when the particular menu is selected
        clr     c1
        clr     c2
        ldi     w1,1
        sts     PWIOInControl,w1
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        ret


PWIO_Left:                                      ;Knob rotated counterclockwise
        sbrc    c2,PWIOC2EditMode
        rjmp    PwL_DecreaseValue
        tst     c1                              ;Move cursor left (not in edit mode)
        breq    PwL_Exit
        dec     c1
        breq    PwL_CheckExtended
        sbrc    c3,PWIOC3EditPW
        rjmp    PwL_Exit
        cpi     c1,5
        brne    PwL_Exit
        clr     c1                              ;Skip PW edit positions if edit isn't allowed
PwL_CheckExtended:
        sbrs    c3,PWIOC3ExtendedMenu           ;Check for multiple submenus
        rjmp    PwL_Exit
        lds     ZL,PWIOExtendedCallback
        lds     ZH,PWIOExtendedCallback+1
        ldi     w1,PWIO_EXTENDLEFT
        icall
        rjmp    PwL_Exit
PwL_DecreaseValue:                              ;Editmode. Decrement pulsewidth
        ldi     w1,5
        sub     w1,c1
        call    Util_Get10expw1_X
        lds     ZL,PWIOAddress
        lds     ZH,PWIOAddress+1
        ld      YL,Z+
        ld      YH,Z+
        call    Util_GetSpeedFactor_w1
PwL_DecreaseLoop:
        sub     YL,XL
        sbc     YH,XH
        brcs    PwL_Wrap
        dec     w1
        brne    PwL_DecreaseLoop
        rjmp    PwL_DecreaseEnd
PwL_Wrap:
        clr     YH
        clr     YL
PwL_DecreaseEnd:
        movw    XH:XL,YH:YL
        call    PWIO_PWInsideBoundaries
        breq    PwL_ValueOk
        lds     XL,PWIOMinPW
        lds     XH,PWIOMinPW+1
PwL_ValueOk:
        in      w1,SREG
        cli
        st      -Z,XH
        st      -Z,XL
        out     SREG,w1
PwL_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        ret


PWIO_Right:                                     ;Knob rotated clockwise
        sbrc    c2,PWIOC2EditMode
        rjmp    PwR_IncreaseValue
        cpi     c1,7                            ;Move cursor right (not in edit mode)
        brsh    PwR_CheckExtended
        mov     w1,c1
        inc     w1
        cpi     w1,1
        brne    PwR_Not1
        sbrs    c3,PWIOC3EditPW
        ldi     w1,6
PwR_Not1:
        cpi     w1,6
        brne    PwR_NoMoreChecks
        sbrs    c3,PWIOC3DisplayAB
        rjmp    PwR_CheckExtended
PwR_NoMoreChecks:
        mov     c1,w1
        rjmp    PwR_Exit
PwR_CheckExtended:
        sbrs    c3,PWIOC3ExtendedMenu           ;Check for multiple submenus
        rjmp    PwR_Exit
        lds     ZL,PWIOExtendedCallback
        lds     ZH,PWIOExtendedCallback+1
        ldi     w1,PWIO_EXTENDRIGHT
        icall
        rjmp    PwR_Exit
PwR_IncreaseValue:                              ;Editmode. Increment pulsewidth
        ldi     w1,5
        sub     w1,c1
        call    Util_Get10expw1_X
        lds     ZL,PWIOAddress
        lds     ZH,PWIOAddress+1
        ld      YL,Z+
        ld      YH,Z+
        call    Util_GetSpeedFactor_w1
PwR_IncreaseLoop:
        add     YL,XL
        adc     YH,XH
        brcs    PwR_Wrap
        dec     w1
        brne    PwR_IncreaseLoop
        rjmp    PwR_IncreaseEnd
PwR_Wrap:
        ser     YH
        ser     YL
PwR_IncreaseEnd:
        movw    XH:XL,YH:YL
        call    PWIO_PWInsideBoundaries
        breq    PwR_ValueOk
        lds     XL,PWIOMaxPW
        lds     XH,PWIOMaxPW+1
PwR_ValueOk:
        in      w1,SREG
        cli
        st      -Z,XH
        st      -Z,XL
        out     SREG,w1
PwR_Exit:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        ret


PWIO_Push:                                      ;Knob pushed
        tst     c1
        brne    PwP_NotMenuExit
        clr     w1                              ;Cursor to the far left: Exit menu
        sts     PWIOInControl,w1
        ldi     w1,EVENT_ReleaseControl
        call    EventQ_PutEventInFront
        rjmp    PwP_Exit
PwP_NotMenuExit:
        cpi     c1,6
        brsh    PwP_SelectAB
        ldi     w1,(1<<PWIOC2EditMode)          ;Cursor in pulsewidth: Toggle edit mode
        eor     c2,w1
        cbr     c2,(1<<PWIOC2BarMode)
        call    LCD_SetCustomCharset1_w1Z
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        rjmp    PwP_Exit
PwP_SelectAB:                                   ;Cursor in output enable area: Set output
        lds     w1,PWIORouterAdr
        cpi     c1,6
        brne    PwP_SelectB
        call    RouterSetOutputA
        rjmp    PwP_SelectDone
PwP_SelectB:
        call    RouterSetOutputB
PwP_SelectDone:
        sbr     c2,(1<<PWIOC2CursorReverse)     ;Force cursor char reversal
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
PwP_Exit:
        ret


PWIO_UpdateDisplay:
        call    Cursor_Stop
        rcall   PWIO_BaseDisplay
        ldi     w3,LCD_CharRightArrow
        ldi     w4,LCD_CharLeftArrow
        sbrs    c2,PWIOC2EditMode
        rjmp    PwUD_NotIncrementMode
        ldi     w3,' '
        ldi     w4,LCD_CharAddSub
PwUD_NotIncrementMode:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        mov     w1,w3
        call    LCD_SendData
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        mov     w1,w4
        call    LCD_SendData
        ret


PWIO_UpdateCursor:
        sbrc    c2,PWIOC2BarMode
        rjmp    PwUC_NoCursor           ; No cursor in bar mode
        mov     w1,c1
        cpi     c1,1
        brlo    PwUC_CursorPosFound
        inc     w1
        cpi     c1,2
        brlo    PwUC_CursorPosFound
        inc     w1
        cpi     c1,6
        brlo    PwUC_CursorPosFound
        subi    w1,-6
PwUC_CursorPosFound:
        ldi     w2,1
        call    LCD_GotoXY
        push    w1
        call    LCD_ReadData_w1
        mov     w3,w1
        pop     w1
        ldi     w4,LCD_CharBlock
        sbrc    c2,PWIOC2EditMode
        rjmp    PwUC_NoBlock
        sbrc    c2,PWIOC2CursorReverse
        rjmp    PwUC_NoBlock
        mov     w4,w3
        ldi     w3,LCD_CharBlock
PwUC_NoBlock:
        call    Cursor_Start
PwUC_NoCursor:
        cbr     c2,(1<<PWIOC2CursorReverse)
        ret


PWIO_ICP:
        ret


PWIO_TimerTick:
        ret


PWIO_ADComplete:
        ret


PWIO_OutputSent:
        ret


PWIO_TachoUpdate:
        ret
        

PWIO_Hold:                                      ;Knob held down: Enable analogue mode (if allowed)
        cbr     c2,(1<<PWIOC2BarMode)
        call    LCD_SetCustomCharset1_w1Z
        sbrs    c3,PWIOC3AllowBarMode
        rjmp    PwH_Exit
        tst     c1
        breq    PwH_Exit
        cpi     c1,6
        brsh    PwH_Exit
        sbr     c2,(1<<PWIOC2EditMode)+(1<<PWIOC2BarMode)
        call    LCD_SetCustomCharset2_w1Z
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
PwH_Exit:
        ret
        

PWIO_BaseDisplay:                       ;Refresh screen contents
        call    LCD_CursorOff
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        lds     ZL,PWIOText1Ptr
        lds     ZH,PWIOText1Ptr+1
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*PWIO_Line2a)
        ldi     ZH,high(2*PWIO_Line2a)
        call    LCD_WriteString_Z
        lds     ZL,PWIOAddress
        lds     ZH,PWIOAddress+1
        ld      XL,Z+
        ld      XH,Z+
        lds     w1,PWIOInControl
        tst     w1
        breq    PwBD_NormalDisplay
        sbrs    c2,PWIOC2BarMode
        rjmp    PwBD_NormalDisplay
        lds     ZL,PWIOBarAddressLow
        lds     ZH,PWIOBarAddressLow+1
        ld      XL,Z+
        ld      XH,Z+
        call    Util_FindBarPosition_All
        push    XL
        lds     ZL,PWIOBarAddressHigh
        lds     ZH,PWIOBarAddressHigh+1
        ld      XL,Z+
        ld      XH,Z+
        call    Util_FindBarPosition_All
        mov     w2,XL
        pop     w1
        call    Util_WriteBar_All
        ldi     w1,' '
        call    LCD_SendData
        rjmp    PwBD_DisplayCommon
PwBD_NormalDisplay:
        call    Util_WritePulseWidth_XY
        ldi     ZL,low(2*PWIO_Line2b)
        ldi     ZH,high(2*PWIO_Line2b)
        call    LCD_WriteString_Z
PwBD_DisplayCommon:
        ldi     w1,LCD_CharSelect
        lds     w2,RouterOutputA
        lds     w3,PWIORouterAdr
        cpse    w2,w3
        ldi     w1,' '
        sbrs    c3,PWIOC3DisplayAB
        ldi     w1,' '
        call    LCD_SendData
        ldi     w1,LCD_CharSelect
        lds     w2,RouterOutputB
;       lds     w3,PWIORouterAdr        Already loaded
        cpse    w2,w3
        ldi     w1,' '
        sbrs    c3,PWIOC3DisplayAB
        ldi     w1,' '
        call    LCD_SendData
        ret

PWIO_Line2a:    .db "  ",0,0
PWIO_Line2b:    .db "   ",0


PWIO_PWInsideBoundaries:                ; Check X for boundaries. Z(flag) set if inside limits
        push    YL
        push    YH
        lds     YL,PWIOMinPW
        lds     YH,PWIOMinPW+1
        cp      XL,YL
        cpc     XH,YH
        brlo    PPWIB_Error
        lds     YL,PWIOMaxPW
        lds     YH,PWIOMaxPW+1
        cp      YL,XL
        cpc     YH,XH
        brlo    PPWIB_Error
        sez
        rjmp    PPWIB_Exit
PPWIB_Error:
        clz
PPWIB_Exit:
        pop     YH
        pop     YL
        ret


PWIO_SetupPWConfigLimit_X:
        call    Config_GetMinPW_X
        sts     PWIOMinPW,XL
        sts     PWIOMinPW+1,XH
        call    Config_GetMaxPW_X
        sts     PWIOMaxPW,XL
        sts     PWIOMaxPW+1,XH
        ret


PWIO_SetupPWNoLimit_w1:
        clr     w1
        sts     PWIOMinPW,w1
        sts     PWIOMinPW+1,w1
        ser     w1
        sts     PWIOMaxPW,w1
        sts     PWIOMaxPW+1,w1
        ret


