;****** Servo Tester 2
;       Rotary include file
;       Rotary knob driver
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    Rotary_InitialSpeed = 20
.equ    Rotary_HoldTreshold = 80        ; = 800 ms


.dseg
RotaryReverse:          .byte 1         ;Swap left-right (config)

RotaryTurnSpeed:        .byte 1         ;Time between last two clicks on encoder (0=slow)

RotaryState:            .byte 1         ;Internal working var
RotaryPushState:        .byte 1         ;Internal working var
RotarySpeedCounter:     .byte 1
RotaryState2:           .byte 1
RotaryHoldCounter:      .byte 1

.cseg

RotaryChange_int:
        push    w1
        in      w1,SREG
        push    w1
        push    w2
        push    w3
        in      w1,PIND
        mov     w2,w1
        lsr     w2
        eor     w2,w1
        sbrc    w2,PD2
        rjmp    RCint_Shifting          ;A and B are different
        lds     w2,RotaryState          ;A and B are equal (knob steady-state)
        eor     w2,w1
        sbrs    w2,PD2
        rjmp    RCint_Exit              ;Same steady-state as before: Ignore
        lds     w2,RotaryState
        sts     RotaryState,w1
        lds     w3,RotaryState2         ;Check for missed step
        tst     w3
        brne    RCint_NoMiss
        rjmp    RCint_Exit              ;Missed direction information: Ignore
RCint_NoMiss:
        clr     w3
        sts     RotaryState2,w3
        lsr     w1
        lsr     w1
        eor     w2,w1
        lds     w1,RotaryReverse
        eor     w2,w1                   ;Correct for left-right reversal (set in config)
        ldi     w1,EVENT_Left
        sbrc    w2,PD0
        ldi     w1,EVENT_Right
        call    EventQ_PutEvent
        lds     w1,RotarySpeedCounter
        sts     RotaryTurnSpeed,w1
        ldi     w1,Rotary_InitialSpeed
        sts     RotarySpeedCounter,w1
        rjmp    RCint_Exit
RCint_Shifting:
        ldi     w3,1
        sts     RotaryState2,w3
        lds     w2,RotaryState
        andi    w2,0xfc
        lsr     w1
        lsr     w1
        andi    w1,0x03
        or      w1,w2
        sts     RotaryState,w1
RCint_Exit:
        pop     w3
        pop     w2
        pop     w1
        out     SREG,w1
        pop     w1
        ret


Rotary_10ms_int_w12:
        in      w1,PIND
        andi    w1,(1<<PD7)
        lds     w2,RotaryPushState
        lsr     w2
        cbr     w2,(1<<PD7)
        or      w1,w2
        sts     RotaryPushState,w1
        andi    w1,0xf0
        cpi     w1,0x10
        brne    R10MS_CheckHold
        ldi     w1,EVENT_Push
        call    EventQ_PutEvent
        rjmp    R10MS_CheckSpeed
R10MS_CheckHold:
        tst     w1
        brne    R10MS_NotHolding
        lds     w1,RotaryHoldCounter
        tst     w1
        breq    R10MS_CheckSpeed
        dec     w1
        sts     RotaryHoldCounter,w1
        brne    R10MS_CheckSpeed
        ldi     w1,EVENT_Hold
        call    EventQ_PutEvent
        rjmp    R10MS_CheckSpeed
R10MS_NotHolding:
        ldi     w1,Rotary_HoldTreshold
        sts     RotaryHoldCounter,w1
R10MS_CheckSpeed:
        lds     w1,RotarySpeedCounter
        dec     w1
        brmi    R10MS_Exit
        sts     RotarySpeedCounter,w1
R10MS_Exit:
;       rcall   RotaryChange_int        ; Fail-safe for fast glitching encoders (not needed?)
        ret
        

Rotary_Init:
        in      w1,DDRD
        cbr     w1,(1<<DDD2)+(1<<DDD3)+(1<<DDD7)        ;Rotary encoder pins input
        out     DDRD,w1
        in      w1,PORTD
        cbr     w1,(1<<PD2)+(1<<PD3)+(1<<PD7)           ;Disable pullup
        out     PORTD,w1
        in      w1,MCUCR
        andi    w1,0xf0
        ori     w1,(1<<ISC10)+(1<<ISC00)
        out     MCUCR,w1
        in      w1,GICR
        ori     w1,(1<<INT1)+(1<<INT0)
        out     GICR,w1
        in      w1,PIND
        sts     RotaryState,w1                          ;Init state
        ser     w1
        sts     RotaryPushState,w1                      ;Init state
        clr     w1
        sts     RotaryTurnSpeed,w1
        sts     RotarySpeedCounter,w1
        sts     RotaryState2,w1
        ldi     w1,Rotary_HoldTreshold
        sts     RotaryHoldCounter,w1
        ret


