;****** Servo Tester 2
;       Main assembler file, version 1.01. Released 5/4 2011
;
;       Copyright (c) 2004-11, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


; r0-r1 reserved for mul results
.def    u1      =r8     ; General purpose low working register
.def    u2      =r9     ; General purpose low working register
.def    u3      =r10    ; General purpose low working register
.def    u4      =r11    ; General purpose low working register
.def    u5      =r12    ; General purpose low working register
.def    u6      =r13    ; General purpose low working register
.def    u7      =r14    ; General purpose low working register
.def    u8      =r15    ; General purpose low working register
.def    w1      =r16    ; General purpose working register
.def    w2      =r17    ; General purpose working register
.def    w3      =r18    ; General purpose working register
.def    w4      =r19    ; General purpose working register
.def    w5      =r20    ; General purpose working register
.def    w6      =r21    ; General purpose working register
.def    c1      =r22    ; Function oriented config register
.def    c2      =r23    ; Function oriented config register
.def    c3      =r24    ; Function oriented config register
.def    c4      =r25    ; Function oriented config register
; r26-31 reserved to X, Y and Z registers


.cseg
.org    0
        jmp     RESET      ; Reset vector
        jmp     EXT_INT0   ; IRQ0
        jmp     EXT_INT1   ; IRQ1
        jmp     TIM2_COMP  ; Timer2 Output Compare Match
        jmp     TIM2_OVF   ; Timer2 Overflow
        jmp     TIM1_CAPT  ; Timer1 Input Capture
        jmp     TIM1_COMPA ; Timer1 Output Compare Match A
        jmp     TIM1_COMPB ; Timer1 Output Compare Match B
        jmp     TIM1_OVF   ; Timer1 Overflow
        jmp     TIM0_OVF   ; Timer0 Overflow
        jmp     SPI_STC    ; SPI Serial Transfer Complete
        jmp     USART_RXC  ; UART RX Complete
        jmp     USART_DRE  ; UDR Empty
        jmp     USART_TXC  ; UART TX Complete
        jmp     ADC_COMP   ; A/D Converter Conversion Complete
        jmp     EE_RDY     ; EEPROM Ready
        jmp     ANA_COMP   ; Analog Comparator
        jmp     TWI
        jmp     EXT_INT2
        jmp     TIM0_COMP
        jmp     SPM_RDY

;**** Interrupt routines

EXT_INT0:   ; IRQ0
        call    RotaryChange_int
        reti

EXT_INT1:   ; IRQ1
        call    RotaryChange_int
        reti

TIM2_COMP:  ; Timer2 Output Compare Match
        call    Timer_CompareMatch_int
        reti
        
TIM2_OVF:   ; Timer2 Overflow
        reti

TIM1_CAPT:  ; Timer1 Input Capture Handler
        call    Pulse_InputCapture_int
        reti

TIM1_COMPA: ;Timer1 Output Compare Match A Handler
        reti

TIM1_COMPB: ;Timer1 Output Compare Match B Handler
        reti

TIM1_OVF:   ; Timer1 Overflow Handler
        call    Pulse_Overflow_int
        reti

TIM0_OVF:   ; Timer0 Overflow Handler
        reti

SPI_STC:    ; SPI Serial Transfer Complete
        reti

USART_RXC:  ; UART RX Complete Handler
        reti

USART_DRE:  ; UDR Empty Handler
        reti

USART_TXC:  ; UART TX Complete Handler
        reti

ADC_COMP:   ; A/D Converter Conversion Complete
        call    ADC_Complete_int
        reti

EE_RDY:     ; EEPROM Ready
        reti

ANA_COMP:   ; Analog Comparator
        reti

TWI:
        reti

EXT_INT2:
        reti

TIM0_COMP:
        reti

SPM_RDY:
        reti



.include "m16def.inc"
.include "config.inc"
.include "eventdef.inc"
.include "eventq.inc"
.include "eventman.inc"
.include "rotary.inc"
.include "util.inc"
.include "lcd.inc"
.include "timer.inc"
.include "router.inc"
.include "pulse.inc"
.include "cursor.inc"
.include "adc.inc"
.include "bcklight.inc"
.include "battery.inc"

.include "pwio.inc"
.include "output1.inc"
.include "output2.inc"
.include "input.inc"
.include "inputavg.inc"
.include "inputstat.inc"
.include "sweep1.inc"
.include "sweep2.inc"
.include "tacho.inc"
.include "idle.inc"
.include "analog.inc"


.cseg

GreetingText1:  .db " Servo Tester 2 ",0,0
GreetingText2:  .db "  Version 1.01  ",0,0



;**** Application init
RESET:
        ldi     w1,low(RAMEND)          ;Init stackpointer
        out     SPL,w1
        ldi     w1,high(RAMEND)
        out     SPH,w1
        ldi     w1,(1<<ACD)             ;Disable analog comparator
        out     ACSR,w1
        clr     w1
        out     DDRA,w1
        out     DDRB,w1
        out     DDRC,w1
        out     DDRD,w1
        ldi     w2,(1<<PA7)+(1<<PA6)+(1<<PA5)+(1<<PA4)+(1<<PA3)+(1<<PA2)
        out     PORTA,w2                ;Pullup on unused pins
        out     PORTB,w1
        ldi     w2,(1<<PC7)+(1<<PC1)+(1<<PC0)
        out     PORTC,w2                ;Pullup on unused pins
        out     PORTD,w1

        call    Config_Init             ;Call first to init saved runtime vars
        call    EventMan_Init           ;Then init everything else
        call    Rotary_Init
        call    LCD_Init
        call    Cursor_Init
        call    Timer_Init
        call    Router_Init
        call    Pulse_Init
        call    ADC_Init
        call    PWIO_Init
        call    Backlight_Init
        call    Battery_Init

        sei                             ;Enable interrupts

        call    Backlight_Activity_r01w12       ;Turn on backlight at startup

        ldi     w1,0                    ;Start of first line
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*GreetingText1)
        ldi     ZH,high(2*GreetingText1)
        call    LCD_WriteString_Z
        ldi     w1,0                    ;Start of second line
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*GreetingText2)
        ldi     ZH,high(2*GreetingText2)
        call    LCD_WriteString_Z

EmptyQueueLoop:                         ;Remove all events from queue
        call    EventQ_GetEvent_w1
        brts    EmptyQueueLoop

        ldi     w4,40                   ; = 4 seconds
GreetingLoop:
        call    EventQ_GetEvent_w1      ;Wait until user activates an input,
        brtc    GreetingLoop            ;or 4 seconds max
        cpi     w1,EVENT_Left
        breq    ExitGreeting
        cpi     w1,EVENT_Right
        breq    ExitGreeting
        cpi     w1,EVENT_Push
        breq    ExitGreeting
        cpi     w1,EVENT_TimerTick
        brne    GreetingLoop
        dec     w4
        brne    GreetingLoop
ExitGreeting:
        ldi     w1,EVENT_Display        ;Force display update
        call    EventQ_PutEvent

Mainloop:                               ;Start main program loop
        call    EventMan_Update         ;Serve next event
        rjmp    Mainloop


