;****** Servo Tester 2
;       Sweep1 include file
;       Sweep 1 menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    Sweep1UP   = 0
.equ    Sweep1DOWN = 1


.dseg

Sweep1Value:            .byte 2
Sweep1Min:              .byte 2
Sweep1Max:              .byte 2
Sweep1Step:             .byte 2
Sweep1Page:             .byte 1
Sweep1Direction:        .byte 1


.cseg

Sweep1_ServiceTable:
        jmp    Sweep1_Init
        jmp    Sweep1_DisplayMenu
        jmp    Sweep1_TakeControl
        jmp    PWIO_Left
        jmp    PWIO_Right
        jmp    PWIO_Push
        jmp    PWIO_UpdateDisplay
        jmp    PWIO_ICP
        jmp    PWIO_TimerTick
        jmp    PWIO_ADComplete
        jmp    PWIO_UpdateCursor
        jmp    Sweep1_OutputSent
        jmp    PWIO_TachoUpdate
        jmp    PWIO_Hold


Sweep1_Init:
        lds     w1,Sweep1Min
        sts     Sweep1Value,w1
        lds     w1,Sweep1Min+1
        sts     Sweep1Value+1,w1
        ldi     w1,Sweep1UP
        sts     Sweep1Direction,w1
        ret


Sweep1_InitPtrs:
        ldi     ZL,low(Sweep1Value)
        ldi     ZH,high(Sweep1Value)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*Sweep1_Page1)
        ldi     ZH,high(2*Sweep1_Page1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        ldi     w1,Router_Sweep1
        sts     PWIORouterAdr,w1
        ldi     ZL,low(Sweep1_ExtendedCallback)
        ldi     ZH,high(Sweep1_ExtendedCallback)
        sts     PWIOExtendedCallback,ZL
        sts     PWIOExtendedCallback+1,ZH
        clr     w1
        sts     Sweep1Page,w1
        call    PWIO_SetupPWConfigLimit_X
        clr     c2
        ldi     c3,(1<<PWIOC3DisplayAB)+(1<<PWIOC3ExtendedMenu)
        ret


Sweep1_DisplayMenu:
        rcall   Sweep1_InitPtrs
        call    PWIO_BaseDisplay
        ret


Sweep1_TakeControl:
        rcall   Sweep1_InitPtrs
        call    PWIO_TakeControl
        ret


Sweep1_OutputSent:
        lds     w1,Sweep1Page
        tst     w1                      ;Only update display on page 1
        brne    S1OS_Exit
        ldi     w1,2
        ldi     w2,1
        call    LCD_GotoXY
        lds     XL,Sweep1Value
        lds     XH,Sweep1Value+1
        call    Util_WritePulseWidth_XY
S1OS_Exit:
        ret



Sweep1_Page1:   .db "  Sweep 1     AB",0,0
Sweep1_Page2:   .db "  Low limit     ",0,0
Sweep1_Page3:   .db "  High limit    ",0,0
Sweep1_Page4:   .db "  Step/speed    ",0,0


Sweep1_UpdateSweep_All:
        lds     ZL,Sweep1Value
        lds     ZH,Sweep1Value+1
        lds     XL,Sweep1Step
        lds     XH,Sweep1Step+1
        lds     w1,Sweep1Direction
        cpi     w1,Sweep1UP
        brne    S1US_Down
        lds     YL,Sweep1Max
        lds     YH,Sweep1Max+1
        add     ZL,XL
        adc     ZH,XH
        brcc    S1US_UpNoWrap
        ser     ZL
        ser     ZH
S1US_UpNoWrap:
        cp      ZL,YL
        cpc     ZH,YH
        brlo    S1US_NotMax
        movw    ZH:ZL,YH:YL
        ldi     w1,Sweep1DOWN
        sts     Sweep1Direction,w1
S1US_NotMax:
        rjmp    S1US_Exit
S1US_Down:
        lds     YL,Sweep1Min
        lds     YH,Sweep1Min+1
        sub     ZL,XL
        sbc     ZH,XH
        brcc    S1US_DownNoWrap
        clr     ZL
        clr     ZH
S1US_DownNoWrap:
        cp      YL,ZL
        cpc     YH,ZH
        brlo    S1US_NotMin
        movw    ZH:ZL,YH:YL
        ldi     w1,Sweep1UP
        sts     Sweep1Direction,w1
S1US_NotMin:
S1US_Exit:
        in      w1,SREG
        cli
        sts     Sweep1Value,ZL
        sts     Sweep1Value+1,ZH
        out     SREG,w1
        ret


Sweep1_ExtendedCallback:
        lds     w2,Sweep1Page
        cpi     w1,PWIO_EXTENDLEFT
        brne    S1EC_Right
        tst     w2
        brne    S1EC_GoLeft
        rjmp    S1EC_Exit
S1EC_GoLeft:
        dec     w2
        rjmp    S1EC_SetupPage
S1EC_Right:
        cpi     w2,3
        brne    S1EC_GoRight
        rjmp    S1EC_Exit
S1EC_GoRight:
        inc     w2
S1EC_SetupPage:
        sts     Sweep1Page,w2
        cpi     w2,0
        brne    S1EC_NotPage1
        ldi     ZL,low(Sweep1Value)
        ldi     ZH,high(Sweep1Value)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*Sweep1_Page1)
        ldi     ZH,high(2*Sweep1_Page1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    PWIO_SetupPWConfigLimit_X
        sbr     c3,(1<<PWIOC3DisplayAB)
        cbr     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3AllowBarMode)
        ldi     c1,0
        cpi     w1,PWIO_EXTENDLEFT
        brne    S1EC_Page1NotLeft
        ldi     c1,7
S1EC_Page1NotLeft:
        rjmp    S1EC_Exit
S1EC_NotPage1:
        cpi     w2,1
        brne    S1EC_NotPage2
        ldi     ZL,low(Sweep1Min)
        ldi     ZH,high(Sweep1Min)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        sts     PWIOBarAddressLow,ZL
        sts     PWIOBarAddressLow+1,ZH
        ldi     ZL,low(Sweep1Max)
        ldi     ZH,high(Sweep1Max)
        sts     PWIOBarAddressHigh,ZL
        sts     PWIOBarAddressHigh+1,ZH
        ldi     ZL,low(2*Sweep1_Page2)
        ldi     ZH,high(2*Sweep1_Page2)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    Config_GetMinPW_X
        sts     PWIOMinPW,XL
        sts     PWIOMinPW+1,XH
        lds     XL,Sweep1Max            ;Make sure min does not exceed max
        lds     XH,Sweep1Max+1
        sts     PWIOMaxPW,XL
        sts     PWIOMaxPW+1,XH
        sbr     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3AllowBarMode)
        cbr     c3,(1<<PWIOC3DisplayAB)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    S1EC_Page2NotLeft
        ldi     c1,5
S1EC_Page2NotLeft:
        rjmp    S1EC_Exit
S1EC_NotPage2:
        cpi     w2,2
        brne    S1EC_NotPage3
        ldi     ZL,low(Sweep1Min)
        ldi     ZH,high(Sweep1Min)
        sts     PWIOBarAddressLow,ZL
        sts     PWIOBarAddressLow+1,ZH
        ldi     ZL,low(Sweep1Max)
        ldi     ZH,high(Sweep1Max)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        sts     PWIOBarAddressHigh,ZL
        sts     PWIOBarAddressHigh+1,ZH
        ldi     ZL,low(2*Sweep1_Page3)
        ldi     ZH,high(2*Sweep1_Page3)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        lds     XL,Sweep1Min            ;Make sure max does not go below min
        lds     XH,Sweep1Min+1
        sts     PWIOMinPW,XL
        sts     PWIOMinPW+1,XH
        call    Config_GetMaxPW_X
        sts     PWIOMaxPW,XL
        sts     PWIOMaxPW+1,XH
        sbr     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3AllowBarMode)
        cbr     c3,(1<<PWIOC3DisplayAB)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    S1EC_Page3NotLeft
        ldi     c1,5
S1EC_Page3NotLeft:
        rjmp    S1EC_Exit
S1EC_NotPage3:
        cpi     w2,3
        brne    S1EC_Exit               ;Unknown page !!
        ldi     ZL,low(Sweep1Step)
        ldi     ZH,high(Sweep1Step)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*Sweep1_Page4)
        ldi     ZH,high(2*Sweep1_Page4)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    PWIO_SetupPWNoLimit_w1
        sbr     c3,(1<<PWIOC3EditPW)
        cbr     c3,(1<<PWIOC3DisplayAB)+(1<<PWIOC3AllowBarMode)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    S1EC_Page4NotLeft
        ldi     c1,5
S1EC_Page4NotLeft:
S1EC_Exit:
        ret

