;****** Servo Tester 2
;       Sweep2 include file
;       Sweep 2 menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    Sweep2UP   = 0
.equ    Sweep2DOWN = 1


.dseg

Sweep2Value:            .byte 2
Sweep2Min:              .byte 2
Sweep2Max:              .byte 2
Sweep2Step:             .byte 2
Sweep2Page:             .byte 1
Sweep2Direction:        .byte 1


.cseg

Sweep2_ServiceTable:
        jmp    Sweep2_Init
        jmp    Sweep2_DisplayMenu
        jmp    Sweep2_TakeControl
        jmp    PWIO_Left
        jmp    PWIO_Right
        jmp    PWIO_Push
        jmp    PWIO_UpdateDisplay
        jmp    PWIO_ICP
        jmp    PWIO_TimerTick
        jmp    PWIO_ADComplete
        jmp    PWIO_UpdateCursor
        jmp    Sweep2_OutputSent
        jmp    PWIO_TachoUpdate
        jmp    PWIO_Hold


Sweep2_Init:
        lds     w1,Sweep2Min
        sts     Sweep2Value,w1
        lds     w1,Sweep2Min+1
        sts     Sweep2Value+1,w1
        ldi     w1,Sweep2UP
        sts     Sweep2Direction,w1
        ret


Sweep2_InitPtrs:
        ldi     ZL,low(Sweep2Value)
        ldi     ZH,high(Sweep2Value)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*Sweep2_Page1)
        ldi     ZH,high(2*Sweep2_Page1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        ldi     w1,Router_Sweep2
        sts     PWIORouterAdr,w1
        ldi     ZL,low(Sweep2_ExtendedCallback)
        ldi     ZH,high(Sweep2_ExtendedCallback)
        sts     PWIOExtendedCallback,ZL
        sts     PWIOExtendedCallback+1,ZH
        clr     w1
        sts     Sweep2Page,w1
        call    PWIO_SetupPWConfigLimit_X
        clr     c2
        ldi     c3,(1<<PWIOC3DisplayAB)+(1<<PWIOC3ExtendedMenu)
        ret


Sweep2_DisplayMenu:
        rcall   Sweep2_InitPtrs
        call    PWIO_BaseDisplay
        ret


Sweep2_TakeControl:
        rcall   Sweep2_InitPtrs
        call    PWIO_TakeControl
        ret


Sweep2_OutputSent:
        lds     w1,Sweep2Page
        tst     w1                      ;Only update display on page 1
        brne    S2OS_Exit
        ldi     w1,2
        ldi     w2,1
        call    LCD_GotoXY
        lds     XL,Sweep2Value
        lds     XH,Sweep2Value+1
        call    Util_WritePulseWidth_XY
S2OS_Exit:
        ret



Sweep2_Page1:   .db "  Sweep 2     AB",0,0
Sweep2_Page2:   .db "  Low limit     ",0,0
Sweep2_Page3:   .db "  High limit    ",0,0
Sweep2_Page4:   .db "  Step/speed    ",0,0


Sweep2_UpdateSweep_All:
        lds     ZL,Sweep2Value
        lds     ZH,Sweep2Value+1
        lds     XL,Sweep2Step
        lds     XH,Sweep2Step+1
        lds     w1,Sweep2Direction
        cpi     w1,Sweep2UP
        brne    S2US_Down
        lds     YL,Sweep2Max
        lds     YH,Sweep2Max+1
        add     ZL,XL
        adc     ZH,XH
        brcc    S2US_UpNoWrap
        ser     ZL
        ser     ZH
S2US_UpNoWrap:
        cp      ZL,YL
        cpc     ZH,YH
        brlo    S2US_NotMax
        movw    ZH:ZL,YH:YL
        ldi     w1,Sweep2DOWN
        sts     Sweep2Direction,w1
S2US_NotMax:
        rjmp    S2US_Exit
S2US_Down:
        lds     YL,Sweep2Min
        lds     YH,Sweep2Min+1
        sub     ZL,XL
        sbc     ZH,XH
        brcc    S2US_DownNoWrap
        clr     ZL
        clr     ZH
S2US_DownNoWrap:
        cp      YL,ZL
        cpc     YH,ZH
        brlo    S2US_NotMin
        movw    ZH:ZL,YH:YL
        ldi     w1,Sweep2UP
        sts     Sweep2Direction,w1
S2US_NotMin:
S2US_Exit:
        in      w1,SREG
        cli
        sts     Sweep2Value,ZL
        sts     Sweep2Value+1,ZH
        out     SREG,w1
        ret


Sweep2_ExtendedCallback:
        lds     w2,Sweep2Page
        cpi     w1,PWIO_EXTENDLEFT
        brne    S2EC_Right
        tst     w2
        brne    S2EC_GoLeft
        rjmp    S2EC_Exit
S2EC_GoLeft:
        dec     w2
        rjmp    S2EC_SetupPage
S2EC_Right:
        cpi     w2,3
        brne    S2EC_GoRight
        rjmp    S2EC_Exit
S2EC_GoRight:
        inc     w2
S2EC_SetupPage:
        sts     Sweep2Page,w2
        cpi     w2,0
        brne    S2EC_NotPage1
        ldi     ZL,low(Sweep2Value)
        ldi     ZH,high(Sweep2Value)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*Sweep2_Page1)
        ldi     ZH,high(2*Sweep2_Page1)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    PWIO_SetupPWConfigLimit_X
        sbr     c3,(1<<PWIOC3DisplayAB)
        cbr     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3AllowBarMode)
        ldi     c1,0
        cpi     w1,PWIO_EXTENDLEFT
        brne    S2EC_Page1NotLeft
        ldi     c1,7
S2EC_Page1NotLeft:
        rjmp    S2EC_Exit
S2EC_NotPage1:
        cpi     w2,1
        brne    S2EC_NotPage2
        ldi     ZL,low(Sweep2Min)
        ldi     ZH,high(Sweep2Min)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        sts     PWIOBarAddressLow,ZL
        sts     PWIOBarAddressLow+1,ZH
        ldi     ZL,low(Sweep2Max)
        ldi     ZH,high(Sweep2Max)
        sts     PWIOBarAddressHigh,ZL
        sts     PWIOBarAddressHigh+1,ZH
        ldi     ZL,low(2*Sweep2_Page2)
        ldi     ZH,high(2*Sweep2_Page2)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    Config_GetMinPW_X
        sts     PWIOMinPW,XL
        sts     PWIOMinPW+1,XH
        lds     XL,Sweep2Max            ;Make sure min does not exceed max
        lds     XH,Sweep2Max+1
        sts     PWIOMaxPW,XL
        sts     PWIOMaxPW+1,XH
        sbr     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3AllowBarMode)
        cbr     c3,(1<<PWIOC3DisplayAB)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    S2EC_Page2NotLeft
        ldi     c1,5
S2EC_Page2NotLeft:
        rjmp    S2EC_Exit
S2EC_NotPage2:
        cpi     w2,2
        brne    S2EC_NotPage3
        ldi     ZL,low(Sweep2Min)
        ldi     ZH,high(Sweep2Min)
        sts     PWIOBarAddressLow,ZL
        sts     PWIOBarAddressLow+1,ZH
        ldi     ZL,low(Sweep2Max)
        ldi     ZH,high(Sweep2Max)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        sts     PWIOBarAddressHigh,ZL
        sts     PWIOBarAddressHigh+1,ZH
        ldi     ZL,low(2*Sweep2_Page3)
        ldi     ZH,high(2*Sweep2_Page3)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        lds     XL,Sweep2Min            ;Make sure max does not go below min
        lds     XH,Sweep2Min+1
        sts     PWIOMinPW,XL
        sts     PWIOMinPW+1,XH
        call    Config_GetMaxPW_X
        sts     PWIOMaxPW,XL
        sts     PWIOMaxPW+1,XH
        sbr     c3,(1<<PWIOC3EditPW)+(1<<PWIOC3AllowBarMode)
        cbr     c3,(1<<PWIOC3DisplayAB)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    S2EC_Page3NotLeft
        ldi     c1,5
S2EC_Page3NotLeft:
        rjmp    S2EC_Exit
S2EC_NotPage3:
        cpi     w2,3
        brne    S2EC_Exit               ;Unknown page !!
        ldi     ZL,low(Sweep2Step)
        ldi     ZH,high(Sweep2Step)
        sts     PWIOAddress,ZL
        sts     PWIOAddress+1,ZH
        ldi     ZL,low(2*Sweep2_Page4)
        ldi     ZH,high(2*Sweep2_Page4)
        sts     PWIOText1Ptr,ZL
        sts     PWIOText1Ptr+1,ZH
        call    PWIO_SetupPWNoLimit_w1
        sbr     c3,(1<<PWIOC3EditPW)
        cbr     c3,(1<<PWIOC3DisplayAB)+(1<<PWIOC3AllowBarMode)
        ldi     c1,1
        cpi     w1,PWIO_EXTENDLEFT
        brne    S2EC_Page4NotLeft
        ldi     c1,5
S2EC_Page4NotLeft:
S2EC_Exit:
        ret

