;****** Servo Tester 2
;       Tacho include file
;       Tachometer menu
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.dseg
TachoCounterValue:      .byte 1
TachoMode:              .byte 1
Tacho100msCounter:      .byte 1
TachoBlades:            .byte 1


.cseg

Tacho_100ms_int_w12:
        push    w3
        in      w3,TCNT0
        in      w1,TIFR
        sbrs    w1,TOV0
        rjmp    T100MSIRQ_NotOverflow
        ldi     w1,(1<<TOV0)
        out     TIFR,w1
        lds     w1,TachoMode
        cpi     w1,1
        breq    T100MSIRQ_AlreadyAtMax
        ldi     w1,1
        sts     TachoMode,w1
        clr     w1
        out     TCNT0,w1
        sts     Tacho100msCounter,w1
        rjmp    T100MSIRQ_Exit
T100MSIRQ_AlreadyAtMax:
        ser     w1
        sts     TachoCounterValue,w1
        clr     w1
        out     TCNT0,w1
        sts     Tacho100msCounter,w1
        ldi     w1,EVENT_TachoUpdate    ;Announce new tacho value
        call    EventQ_PutEvent
        rjmp    T100MSIRQ_Exit
T100MSIRQ_NotOverflow:
        lds     w1,Tacho100msCounter
        inc     w1
        sts     Tacho100msCounter,w1
        lds     w2,TachoMode
        cp      w1,w2
        brlo    T100MSIRQ_Exit
        cpi     w3,20
        brsh    T100MSIRQ_NotTimeExtend
        cpi     w1,1
        brne    T100MSIRQ_NotTimeExtend
        ldi     w1,10
        sts     TachoMode,w1
        rjmp    T100MSIRQ_Exit
T100MSIRQ_NotTimeExtend:
        sts     TachoCounterValue,w3
        clr     w1
        out     TCNT0,w1
        sts     Tacho100msCounter,w1
        ldi     w1,EVENT_TachoUpdate    ;Announce new tacho value
        call    EventQ_PutEvent
T100MSIRQ_Exit:
        pop     w3
        ret


Tacho_Init:
        cbi     DDRB,PB0
        sbi     PORTB,PB0
        clr     w1
        sts     TachoCounterValue,w1
        sts     Tacho100msCounter,w1
        out     TCNT0,w1
        ldi     w1,10
        sts     TachoMode,w1
        ldi     w1,(1<<CS02)+(1<<CS01)+(1<<CS00)
        out     TCCR0,w1                ;External clock at T0 pin. Rising edge
        in      w1,TIMSK
        cbr     w1,(1<<OCIE0)+(1<<TOIE0)
        out     TIMSK,w1
        ldi     w1,(1<<TOV0)
        out     TIFR,w1
        ret



Tacho_ServiceTable:
        jmp    Tacho_Init
        jmp    Tacho_DisplayMenu
        jmp    Tacho_TakeControl
        jmp    Tacho_Left
        jmp    Tacho_Right
        jmp    Tacho_Push
        jmp    Tacho_UpdateDisplay
        jmp    Tacho_ICP
        jmp    Tacho_TimerTick
        jmp    Tacho_ADComplete
        jmp    Tacho_UpdateCursor
        jmp    Tacho_OutputSent
        jmp    Tacho_TachoUpdate
        jmp    Tacho_Hold



Tacho_DisplayMenu:
        rcall   Tacho_DisplayCommon
        ret


Tacho_TakeControl:
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        ret
        
        
Tacho_Left:
        lds     w1,TachoBlades
        dec     w1
        cpi     w1,2                            ;Min 2 blade prop
        brsh    TL_NotLow
        ldi     w1,2
TL_NotLow:
        sts     TachoBlades,w1
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        ret
        
        
Tacho_Right:
        lds     w1,TachoBlades
        inc     w1
        cpi     w1,5                            ;Max 4 blade prop
        brlo    TR_NotHigh
        ldi     w1,4
TR_NotHigh:
        sts     TachoBlades,w1
        ldi     w1,EVENT_Display                ;Request display update
        call    EventQ_PutEventIfNew
        ldi     w1,EVENT_UpdateCursor
        call    EventQ_PutEventIfNew
        ret
        
        
Tacho_Push:
        ldi     w1,EVENT_ReleaseControl
        call    EventQ_PutEventInFront
        ret
        
        
Tacho_UpdateDisplay:
        call    Cursor_Stop
        rcall   Tacho_DisplayCommon
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     w1,LCD_CharAddSub
        call    LCD_SendData
        ret
        
        
Tacho_ICP:
        ret
        
        
Tacho_TimerTick:
        ret


Tacho_ADComplete:
        ret
        
        
Tacho_UpdateCursor:
        ldi     w1,9
        ldi     w2,0
        call    LCD_GotoXY
        push    w1
        call    LCD_ReadData_w1
        mov     w3,w1
        pop     w1
        ldi     w4,LCD_CharBlock
        call    Cursor_Start
        ret
        
        
Tacho_OutputSent:
        ret
        
        
Tacho_TachoUpdate:
        ldi     w1,2
        ldi     w2,1
        call    LCD_GotoXY
        rcall   Tacho_DisplayRPM
        ret


Tacho_Hold:
        ret
        

Tacho_DisplayCommon:
        ldi     w1,0
        ldi     w2,0
        call    LCD_GotoXY
        ldi     ZL,low(2*Tacho_Line1a)
        ldi     ZH,high(2*Tacho_Line1a)
        call    LCD_WriteString_Z
        lds     w1,TachoBlades
        ori     w1,'0'
        call    LCD_SendData
        ldi     ZL,low(2*Tacho_Line1b)
        ldi     ZH,high(2*Tacho_Line1b)
        call    LCD_WriteString_Z
        ldi     w1,0
        ldi     w2,1
        call    LCD_GotoXY
        ldi     ZL,low(2*Tacho_Line2a)
        ldi     ZH,high(2*Tacho_Line2a)
        call    LCD_WriteString_Z
        rcall   Tacho_DisplayRPM
        ldi     ZL,low(2*Tacho_Line2b)
        ldi     ZH,high(2*Tacho_Line2b)
        call    LCD_WriteString_Z
        ret

Tacho_Line1a:   .db "  Tacho  ",0
Tacho_Line1b:   .db " blade",0,0
Tacho_Line2a:   .db "  ",0,0
Tacho_Line2b:   .db " rpm     ",0


Tacho_DisplayRPM:
        lds     w1,TachoCounterValue
        lds     w3,TachoBlades
        cpi     w3,3
        brlo    TUD_2blade
        breq    TUD_3blade
        ldi     w2,15                           ;4-blade prop
        rjmp    TUD_BladeFound
TUD_3blade:
        ldi     w2,20                           ;3-blade prop
        rjmp    TUD_BladeFound
TUD_2blade:
        ldi     w2,30                           ;2-blade prop
TUD_BladeFound:
        mul     w1,w2
        movw    XH:XL,r1:r0
        lds     w1,TachoMode
        cpi     w1,1
        brne    TUD_DisplayRPM
        ldi     YL,low(10)
        ldi     YH,high(10)
        call    mul16x16_32_w                   ;Fast mode -> x10
        movw    XH:XL,w2:w1
        or      w3,w4
        breq    TUD_DisplayRPM
        ser     XL                              ;Max exceeded
        ser     XH
TUD_DisplayRPM:
        call    Util_WriteU16_5d_XY
        ret


