;****** Servo Tester 2
;       Util include file
;       Various useful utilities
;
;       Copyright (c) 2004, Mikael Ejberg Pedersen
;       Only for non-profit use. See a more detailed license
;       description in the user manual or at the Servo Tester 2
;       homepage: http://www.ejberg.dk/servotst2


.equ    Pulse_BarLength = 11            ; 11 characters


Util_Delayus:                           ;Delays execution for w1 us (+ a bit)
        push    w1                      ;Only valid for 10 MHz xtal
UDus_Loop:
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        dec     w1
        brne    UDus_Loop
        pop     w1
        ret


Util_Delayms_X:                         ;Delays execution for X ms (+ a bit)
        push    w1
        ldi     w1,250
UDms_Loop:
        rcall   Util_Delayus
        rcall   Util_Delayus
        rcall   Util_Delayus
        rcall   Util_Delayus
        sbiw    XH:XL,1
        brne    UDms_Loop
        pop     w1
        ret


Util_FindDigit_w1X:
        clr     w1              ;Returns X DIV Y in w1
UFD_Loop:                       ;and X MOD Y in X
        cp      XL,YL
        cpc     XH,YH
        brlo    UFD_Exit
        inc     w1
        sub     XL,YL
        sbc     XH,YH
        rjmp    UFD_Loop
UFD_Exit:
        ret


Util_Write4digit_w12XY:         ;Pulse width to display in X (max 9999), digits before decimal point in w2
        ldi     YL,LOW(1000)    ;T=0 for blanking leading 0's. T=1 to show all 0's
        ldi     YH,HIGH(1000)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brts    UW4D_WriteDigit
        set
        cpi     w1,'0'
        brne    UW4D_WriteDigit
        cpi     w2,2
        brlt    UW4D_WriteDigit
        ldi     w1,' '
        clt
UW4D_WriteDigit:
        call    LCD_SendData
        dec     w2
        brne    UW4D_SkipDecimalPoint
        ldi     w1,'.'
        call    LCD_SendData
UW4D_SkipDecimalPoint:
Util_Write3digit_w12XY:         ;Pulse width to display in X (max 999), digits before decimal point in w2
        ldi     YL,LOW(100)
        ldi     YH,HIGH(100)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brts    UW3D_WriteDigit
        set
        cpi     w1,'0'
        brne    UW3D_WriteDigit
        cpi     w2,2
        brlt    UW3D_WriteDigit
        ldi     w1,' '
        clt
UW3D_WriteDigit:
        call    LCD_SendData
        dec     w2
        brne    UW3D_SkipDecimalPoint
        ldi     w1,'.'
        call    LCD_SendData
UW3D_SkipDecimalPoint:
Util_Write2digit_w12XY:         ;Pulse width to display in X (max 99), digits before decimal point in w2
        ldi     YL,LOW(10)
        ldi     YH,HIGH(10)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brts    UW2D_WriteDigit
        set
        cpi     w1,'0'
        brne    UW2D_WriteDigit
        cpi     w2,2
        brlt    UW2D_WriteDigit
        ldi     w1,' '          ;No need to clear T here, as the last digit must be shown
UW2D_WriteDigit:
        call    LCD_SendData
        dec     w2
        brne    UW2D_SkipDecimalPoint
        ldi     w1,'.'
        call    LCD_SendData
UW2D_SkipDecimalPoint:
        mov     w1,XL
        ori     w1,'0'
        call    LCD_SendData
        ret


Util_WritePulseWidth_XY:        ;Pulse width to display in X
        push    w1
        ldi     YL,LOW(10000)
        ldi     YH,HIGH(10000)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        call    LCD_SendData
        ldi     w1,'.'
        call    LCD_SendData
        ldi     YL,LOW(1000)
        ldi     YH,HIGH(1000)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        call    LCD_SendData
        ldi     YL,LOW(100)
        ldi     YH,HIGH(100)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        call    LCD_SendData
        ldi     YL,LOW(10)
        ldi     YH,HIGH(10)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        call    LCD_SendData
        mov     w1,XL
        ori     w1,'0'
        call    LCD_SendData
        ldi     w1,' '
        call    LCD_SendData
        ldi     w1,'m'
        call    LCD_SendData
        ldi     w1,'s'
        call    LCD_SendData
        pop     w1
        ret


Util_FindBarPosition_All:               ; Y = Pulse width to find position for
        movw    YH:YL,XH:XL             ; Result in XL
        call    Config_GetMinPW_X
        sub     YL,XL                   ; Y = Pulsewidth - MinPulsewidth
        sbc     YH,XH
        movw    ZH:ZL,XH:XL             ; Z = MinPulsewidth
        ldi     XL,5*Pulse_BarLength
        clr     XH
        call    mul16x16_32_w           ; X*Y
        call    Config_GetMaxPW_X
        sub     XL,ZL                   ; X = MaxPulsewidth - MinPulsewidth
        sbc     XH,ZH
        movw    ZH:ZL,XH:XL             ; Z = Max - Min
        movw    XH:XL,w2:w1             ; YH:YL:XH:XL = (Pulse - Min) * 5 * BarLength
        movw    YH:YL,w4:w3
        call    div32_16u_uw1XYZ        ; XL = (Pulse - Min) * 5 * BarLength / (Max - Min)
        cpi     XL,5*Pulse_BarLength
        brlo    UFBP_ResultNotHigh
        ldi     XL,5*Pulse_BarLength-1
UFBP_ResultNotHigh:
        ret


Util_Clearu_u:
        clr     u1
        clr     u2
        clr     u3
        clr     u4
        clr     u5
        clr     u6
        clr     u7
        clr     u8
        ret


Util_IncrementPixelCounter:
        inc     w4
        lsr     w3
        brcc    UIPC_Exit
        ldi     w3,0x10
        tst     u2
        brne    UIPC_DefineChar
        push    w1
        ldi     w1,LCD_CharTopLine
        tst     u1
        brne    UIPC_DisplayChar
        ldi     w1,LCD_CharBottomLine
UIPC_DisplayChar:
        call    LCD_SendData
        pop     w1
        rjmp    UIPC_ClearAndExit
UIPC_DefineChar:
        push    w1
        mov     w1,w5
        call    LCD_SetCustomChar
        call    LCD_SendData            ;Write custom char
        pop     w1
        inc     w5
UIPC_ClearAndExit:
        rcall   Util_Clearu_u
UIPC_Exit:
        ret
        

Util_SetVerticalRow_u:
        or      u1,w3
        or      u2,w3
        or      u3,w3
        or      u4,w3
        or      u5,w3
        or      u6,w3
        or      u7,w3
        ret
        
        
Util_WriteBar_All:                      ;w1 and w2 = Transition edge positions (in pixels)
        ldi     w3,0x10                 ;Bit mask
        clr     w4                      ;Pixel counter
        ldi     w5,LCD_FirstUserChar    ;User defined character counter
        clr     w6                      ;Top flag
        rcall   Util_Clearu_u
UWB_Loop:
        tst     w6
        brne    UWB_SetTopPixel
        or      u7,w3
        rjmp    UWB_TestForTransition
UWB_SetTopPixel:
        or      u1,w3
UWB_TestForTransition:
        cp      w4,w1
        brne    UWB_TestForTransition2
        ser     w6
        rcall   Util_SetVerticalRow_u
UWB_TestForTransition2:
        cp      w4,w2
        brne    UWB_Increment
        clr     w6
        rcall   Util_SetVerticalRow_u
UWB_Increment:
        rcall   Util_IncrementPixelCounter
        cpi     w4,5*Pulse_BarLength
        brne    UWB_Loop
        ret



Util_WriteU8_3d_XY:     ;Byte to display in XL
        push    w1
        set             ;Leading space
        clr     XH
        rjmp    UWU16_Entry3d

Util_WriteU8_2d00_XY:   ;Byte to display in XL
        push    w1
        clt             ;Leading '0'
        clr     XH
        rjmp    UWU16_Entry2d

Util_WriteU8_1d_XY:     ;Byte to display in XL
        push    w1
        clr     XH
        rjmp    UWU16_Entry1d

Util_WriteU16_5d_XY:    ;Word to display in X
        push    w1
        set             ;Leading space
        ldi     YL,LOW(10000)
        ldi     YH,HIGH(10000)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brtc    UWU16_Not0_1
        clt
        cpi     w1,'0'
        brne    UWU16_Not0_1
        set
        ldi     w1,' '
UWU16_Not0_1:
        call    LCD_SendData
        ldi     YL,LOW(1000)
        ldi     YH,HIGH(1000)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brtc    UWU16_Not0_2
        clt
        cpi     w1,'0'
        brne    UWU16_Not0_2
        set
        ldi     w1,' '
UWU16_Not0_2:
        call    LCD_SendData
UWU16_Entry3d:
        ldi     YL,LOW(100)
        ldi     YH,HIGH(100)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brtc    UWU16_Not0_3
        clt
        cpi     w1,'0'
        brne    UWU16_Not0_3
        set
        ldi     w1,' '
UWU16_Not0_3:
        call    LCD_SendData
UWU16_Entry2d:
        ldi     YL,LOW(10)
        ldi     YH,HIGH(10)
        rcall   Util_FindDigit_w1X
        ori     w1,'0'
        brtc    UWU16_Not0_4
        cpi     w1,'0'
        brne    UWU16_Not0_4
        ldi     w1,' '
UWU16_Not0_4:
        call    LCD_SendData
UWU16_Entry1d:
        mov     w1,XL
        ori     w1,'0'
        call    LCD_SendData
        pop     w1
        ret


Util_10expTable:
        .dw     1,10,100,1000,10000

Util_Get10expw1_X:
        push    w1
        push    ZL
        push    ZH
        ldi     ZL,low(2*Util_10expTable)
        ldi     ZH,high(2*Util_10expTable)
        lsl     w1
        add     ZL,w1
        clr     w1
        adc     ZH,w1
        lpm     XL,Z+
        lpm     XH,Z
        pop     ZH
        pop     ZL
        pop     w1
        ret
        
        
Util_SpeedTranslationTable:
;       .db     1,1,1,1,1,2,2,2,2,3,3,4,5,6,7,8,10
        .db     1,1,1,1,1,1,1,1,1,1,2,2,3,3,4,4,5,5

Util_GetSpeedFactor_w1:
        push    ZL
        push    ZH
        call    Config_ExpoInc_w1
        brne    UGSF_NoExpo
        lds     w1,RotaryTurnSpeed
        cpi     w1,17
        brlo    UGSF_NotHighLimit
        ldi     w1,16
UGSF_NotHighLimit:
        ldi     ZL,low(2*Util_SpeedTranslationTable)
        ldi     ZH,high(2*Util_SpeedTranslationTable)
        add     ZL,w1
        clr     w1
        adc     ZH,w1
        lpm     w1,Z
        rjmp    UGSF_Exit
UGSF_NoExpo:
        ldi     w1,1
UGSF_Exit:
        pop     ZH
        pop     ZL
        ret


Util_Speed2TranslationTable:
        .db     1,1,1,1,1,1,1,1,1,1,10,10,10,10,10,10,100,100

Util_GetSpeed2Factor_w1:
        push    ZL
        push    ZH
        lds     w1,RotaryTurnSpeed
        cpi     w1,17
        brlo    UGS2F_NotHighLimit
        ldi     w1,16
UGS2F_NotHighLimit:
        ldi     ZL,low(2*Util_Speed2TranslationTable)
        ldi     ZH,high(2*Util_Speed2TranslationTable)
        add     ZL,w1
        clr     w1
        adc     ZH,w1
        lpm     w1,Z
        pop     ZH
        pop     ZL
        ret


;***************************************************************************
;*
;* "div32_16u" - 32/16 Bit Unsigned Division
;*
;* This subroutine divide a 32 bit integer by a 16 bit integer 
;* "YH:YL:XH:XL" (dividend) and "ZH:ZL" (divisor). 
;* The result is placed in "YH:YL:XH:XL" and the remainder in
;* "u4:u3:u2:u1".
;*  
;* Number of words      :30
;* Number of cycles     :652/748 (Min/Max)
;*
;***************************************************************************

;***** Code

div32_16u_uw1XYZ:
        clr     u5
        clr     u1              ;clear remainder Low byte
        clr     u2
        clr     u3
        sub     u4,u4   ;clear remainder High byte and carry
        ldi     w1,33           ;init loop counter
d32_16u_1:      
        rol     XL              ;shift left dividend
        rol     XH
        rol     YL
        rol     YH
        dec     w1              ;decrement counter
        brne    d32_16u_2               ;if done
        ret                             ;    return
d32_16u_2:      
        rol     u1              ;shift dividend into remainder
        rol     u2
        rol     u3
        rol     u4
        sub     u1,ZL   ;remainder = remainder - divisor
        sbc     u2,ZH
        sbc     u3, u5
        sbc     u4, u5
        brcc    d32_16u_3               ;if result negative
        add     u1,ZL   ;   restore remainder
        adc     u2,ZH
        adc     u3, u5
        adc     u4, u5
        clc                             ;   clear carry to be shifted into result
        rjmp    d32_16u_1               ;else
d32_16u_3:      
        sec                             ;   set carry to be shifted into result
        rjmp    d32_16u_1


;******************************************************************************
;*
;* FUNCTION
;*      mul16x16_32
;* DECRIPTION
;*      Unsigned multiply of two 16bits numbers with 32bits result.
;* USAGE
;*      w4:w3:w2:w1 = XH:XL * YH:YL
;* STATISTICS
;*      Cycles :        17 + ret
;*      Words :         13 + ret
;* NOTE
;*      Full orthogonality i.e. any register pair can be used as long as
;*      the 32bit result and the two operands does not share register pairs.
;*      The routine is non-destructive to the operands.
;*
;******************************************************************************

mul16x16_32_w:
        clr     w5
        mul     XH,YH           ; ah * bh
        movw    w4:w3,r1:r0
        mul     XL,YL           ; al * bl
        movw    w2:w1,r1:r0
        mul     XH,YL           ; ah * bl
        add     w2,r0
        adc     w3,r1
        adc     w4,w5
        mul     YH,XL           ; bh * al
        add     w2,r0
        adc     w3,r1
        adc     w4,w5
        ret


